@@ -20,8 +20,10 @@
 
 #define P_(s) (s)   // FIXME -> gettext
 
-static gboolean gx_hslider_expose (GtkWidget *widget, GdkEventExpose *event);
-static void gx_hslider_size_request (GtkWidget *widget, GtkRequisition *requisition);
+static gboolean gx_hslider_draw (GtkWidget *widget, cairo_t *cr);
+static void gx_hslider_get_preferred_width (GtkWidget *widget, gint *min_width, gint *natural_width);
+static void gx_hslider_get_preferred_height (GtkWidget *widget, gint *min_height, gint *natural_height);
+static void gx_hslider_size_request (GtkWidget *widget, gint *width, gint *height);
 static gboolean gx_hslider_button_press (GtkWidget *widget, GdkEventButton *event);
 static gboolean gx_hslider_button_release (GtkWidget *widget, GdkEventButton *event);
 static gboolean gx_hslider_pointer_motion (GtkWidget *widget, GdkEventMotion *event);
@@ -32,15 +34,16 @@ static void gx_hslider_render_pixbuf (GtkWidget *widget);
 
 G_DEFINE_TYPE(GxHSlider, gx_hslider, GX_TYPE_REGLER);
 
-#define get_stock_id(widget) (GX_HSLIDER_CLASS(GTK_OBJECT_GET_CLASS(widget))->stock_id)
-#define get_mouse_scale_factor(widget) (GX_HSLIDER_CLASS(GTK_OBJECT_GET_CLASS(widget))->mouse_scale_factor)
+#define get_stock_id(widget) (GX_HSLIDER_CLASS(GTK_WIDGET_GET_CLASS(widget))->stock_id)
+#define get_mouse_scale_factor(widget) (GX_HSLIDER_CLASS(GTK_WIDGET_GET_CLASS(widget))->mouse_scale_factor)
 
 static void gx_hslider_class_init(GxHSliderClass *klass)
 {
 	GtkWidgetClass *widget_class = (GtkWidgetClass*) klass;
 
-	widget_class->expose_event = gx_hslider_expose;
-	widget_class->size_request = gx_hslider_size_request;
+	widget_class->draw = gx_hslider_draw;
+	widget_class->get_preferred_width = gx_hslider_get_preferred_width;
+	widget_class->get_preferred_height = gx_hslider_get_preferred_height;
 	widget_class->button_press_event   = gx_hslider_button_press;
     widget_class->button_release_event = gx_hslider_button_release;
 	widget_class->motion_notify_event  = gx_hslider_pointer_motion;
@@ -52,53 +55,78 @@ static void gx_hslider_class_init(GxHSliderClass *klass)
 		widget_class,
 		g_param_spec_int("slider-width",P_("size of slider"),
 		                   P_("Width of movable part of vslider"),
-		                 0, 100, 20, GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
+		                 0, 100, 36, GParamFlags(G_PARAM_READABLE|G_PARAM_STATIC_STRINGS)));
+	gtk_widget_class_set_css_name(widget_class, "gx-hslider");
 }
 
-static void gx_hslider_size_request (GtkWidget *widget, GtkRequisition *requisition)
+static void gx_hslider_get_preferred_width (GtkWidget *widget, gint *min_width, gint *natural_width)
+{
+	gint width, height;
+	gx_hslider_size_request(widget, &width, &height);
+
+	if (min_width) {
+		*min_width = width;
+	}
+	if (natural_width) {
+		*natural_width = width;
+	}
+}
+
+static void gx_hslider_get_preferred_height (GtkWidget *widget, gint *min_height, gint *natural_height)
+{
+	gint width, height;
+	gx_hslider_size_request(widget, &width, &height);
+
+	if (min_height) {
+		*min_height = height;
+	}
+	if (natural_height) {
+		*natural_height = height;
+	}
+}
+
+static void gx_hslider_size_request (GtkWidget *widget, gint *width, gint *height)
 {
 	g_assert(GX_IS_HSLIDER(widget));
     GxHSlider *slider   = GX_HSLIDER(widget);
-	requisition->width  = slider->width;
-	requisition->height = slider->height;
-	_gx_regler_calc_size_request(GX_REGLER(widget), requisition);
+	*width  = slider->width;
+	*height = slider->height;
+	_gx_regler_calc_size_request(GX_REGLER(widget), width, height, TRUE);
 }
 
-static gboolean gx_hslider_expose(GtkWidget *widget, GdkEventExpose *event)
+static gboolean gx_hslider_draw(GtkWidget *widget, cairo_t *cr)
 {
 	g_assert(GX_IS_HSLIDER(widget));
     GxHSlider *slider = GX_HSLIDER(widget);
-    GtkAllocation allocation;
-    gtk_widget_get_allocation(widget, &allocation);
-    int x = allocation.x;
-    int y = allocation.y;
     slider->image_rect.x = slider->image_rect.y = 0;
 	GdkRectangle  value_rect;
     gdouble slstate = _gx_regler_get_step_pos(GX_REGLER(widget), slider->width - slider->slider_width);
-	_gx_regler_get_positions(GX_REGLER(widget), &slider->image_rect, &value_rect);
+    _gx_regler_get_positions(GX_REGLER(widget), &slider->image_rect, &value_rect, false);
+    int x = slider->image_rect.x;
+    int y = slider->image_rect.y;
 	if (gtk_widget_has_focus(widget)) {
-		gtk_paint_focus(gtk_widget_get_style(widget), gtk_widget_get_window(widget), GTK_STATE_NORMAL, NULL, widget, NULL,
-                        x, y, slider->width, slider->height);
+		gtk_render_focus(gtk_widget_get_style_context(widget), cr,
+						 x, y, slider->width, slider->height);
 	}
-	cairo_t *cr = gdk_cairo_create(gtk_widget_get_window(widget));
     // background
 	gdk_cairo_set_source_pixbuf (cr, slider->image, x, y);
 	cairo_rectangle(cr, x, y, slider->width, slider->height);
 	cairo_fill(cr);
     //slider
-    int sx = gtk_widget_get_state(widget) ? slider->slider_width : 0;
+    int sx = (gtk_widget_get_state_flags(widget) & GTK_STATE_FLAG_PRELIGHT) ? slider->slider_width : 0;
     gdk_cairo_set_source_pixbuf (cr, slider->image, x - (slider->width - slstate) - sx, y);
     cairo_rectangle(cr, x + slstate, y, slider->slider_width, slider->height);
     cairo_fill(cr);
-	_gx_regler_display_value(GX_REGLER(widget), &value_rect);
-	cairo_destroy (cr);
+	_gx_regler_display_value(GX_REGLER(widget), cr, &value_rect);
 	return FALSE;
 }
 
 static gboolean gx_hslider_enter_in (GtkWidget *widget, GdkEventCrossing *event)
 {
-    if (gtk_widget_get_state(widget) == GTK_STATE_NORMAL)
-        gtk_widget_set_state(widget, GTK_STATE_PRELIGHT);
+    GtkStateFlags flags = gtk_widget_get_state_flags(widget);
+    if (!(flags & GTK_STATE_FLAG_PRELIGHT)) {
+        gtk_widget_set_state_flags(widget, GTK_STATE_FLAG_PRELIGHT, FALSE);
+    }
     GX_HSLIDER(widget)->hover = 1;
     gx_hslider_set_pointer(widget, NULL);
 	return TRUE;
@@ -106,8 +134,10 @@ static gboolean gx_hslider_enter_in (GtkWidget *widget, GdkEventCrossing *event)
 
 static gboolean gx_hslider_leave_out (GtkWidget *widget, GdkEventCrossing *event)
 {
-    if (gtk_widget_get_state(widget) == GTK_STATE_PRELIGHT)
-        gtk_widget_set_state(widget, GTK_STATE_NORMAL);
+    GtkStateFlags flags = gtk_widget_get_state_flags(widget);
+    if (flags & GTK_STATE_FLAG_PRELIGHT) {
+        gtk_widget_unset_state_flags(widget, GTK_STATE_FLAG_PRELIGHT);
+    }
     GX_HSLIDER(widget)->hover = 0;
     gx_hslider_set_pointer(widget, NULL);
 	return TRUE;
@@ -117,12 +147,8 @@ static gboolean slider_set_from_pointer(GtkWidget *widget, int state, gdouble x,
 {
     GxHSlider *slider = GX_HSLIDER(widget);
 	GdkRectangle value_rect;
-	GtkAllocation allocation;
-	gtk_widget_get_allocation(widget, &allocation);
     slider->image_rect.x = slider->image_rect.y = 0;
-    _gx_regler_get_positions(GX_REGLER(widget), &slider->image_rect, &value_rect);
-	x += allocation.x;
-	y += allocation.y;
+    _gx_regler_get_positions(GX_REGLER(widget), &slider->image_rect, &value_rect, false);
 	if (!drag) {
 		if (_gx_regler_check_display_popup(GX_REGLER(widget), &slider->image_rect, &value_rect, event)) {
 			return FALSE;
@@ -160,7 +186,7 @@ static gboolean gx_hslider_button_press (GtkWidget *widget, GdkEventButton *even
 	gtk_widget_grab_focus(widget);
 	if (slider_set_from_pointer(widget, event->state, event->x, event->y, FALSE, event->button, event)) {
 		gtk_grab_add(widget);
-        gtk_widget_set_state(widget, GTK_STATE_ACTIVE);
+        gtk_widget_set_state_flags(widget, GTK_STATE_FLAG_ACTIVE, FALSE);
         gx_hslider_set_pointer(widget, NULL);
 	}
 	return FALSE;
@@ -173,8 +199,12 @@ static gboolean gx_hslider_button_release (GtkWidget *widget, GdkEventButton *ev
 		return FALSE;
 	}
     gtk_grab_remove(widget);
-    gtk_widget_set_state(widget, GX_HSLIDER(widget)->hover ? GTK_STATE_PRELIGHT : GTK_STATE_NORMAL);
-	return TRUE;
+    if (GX_HSLIDER(widget)->hover) {
+        gtk_widget_set_state_flags(widget, GTK_STATE_FLAG_PRELIGHT, FALSE);
+    } else {
+        gtk_widget_unset_state_flags(widget, GTK_STATE_FLAG_PRELIGHT);
+    }
+    return TRUE;
 }
 
 static gboolean gx_hslider_pointer_motion(GtkWidget *widget, GdkEventMotion *event)
@@ -193,16 +223,17 @@ static void gx_hslider_init(GxHSlider *hslider)
     GtkWidget *widget = GTK_WIDGET(hslider);
     hslider->hover        = 0;
     gx_hslider_render_pixbuf(widget);
-    g_signal_connect ( GTK_OBJECT (widget), "style-set",
+    g_signal_connect ( widget, "style-updated",
                      G_CALLBACK (gx_hslider_render_pixbuf), NULL);
 }
 
 static void gx_hslider_set_pointer (GtkWidget *widget, GdkEventMotion *event)
 {
     GxHSlider *slider = GX_HSLIDER(widget);
-    GdkCursor *cur = gdk_cursor_new(GDK_HAND2);
+    GdkDisplay *disp = gtk_widget_get_display(widget);
+    GdkCursor *cur = gdk_cursor_new_for_display(disp, GDK_HAND2);
     gdouble slstate = _gx_regler_get_step_pos(GX_REGLER(widget), slider->width - slider->slider_width);
-    if (gtk_widget_get_state(widget) == GTK_STATE_ACTIVE
+    if (gtk_widget_get_state_flags(widget) & GTK_STATE_FLAG_ACTIVE
     or (event and event->x > slstate and event->x < slstate + slider->slider_width))
         gdk_window_set_cursor(GDK_WINDOW(gtk_widget_get_window(widget)), cur);
     else
@@ -213,10 +244,13 @@ static void gx_hslider_render_pixbuf (GtkWidget *widget)
 {
     GxHSlider *hslider = GX_HSLIDER(widget);
     gtk_widget_style_get(widget, "slider-width", &hslider->slider_width, NULL);
-    hslider->image        = gtk_widget_render_icon(widget, get_stock_id(widget), GtkIconSize(-1), NULL);
-    hslider->width        = gdk_pixbuf_get_width(hslider->image) - hslider->slider_width * 2;
-    hslider->height       = gdk_pixbuf_get_height(hslider->image);
+    hslider->image = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
+                                              get_stock_id(widget), -1,
+                                              GTK_ICON_LOOKUP_GENERIC_FALLBACK, nullptr);
+    hslider->width = gdk_pixbuf_get_width(hslider->image) - hslider->slider_width * 2;
+    hslider->height = gdk_pixbuf_get_height(hslider->image);
     GdkRectangle rect;
+    rect.x = rect.y = 0;
     rect.width  = hslider->width;
     rect.height = hslider->height;
     hslider->image_rect = rect;
