@@ -25,7 +25,7 @@
 #include <lv2.h>
 
 #define GXPLUGIN_URI "http://guitarix.sourceforge.net/plugins/gx_aclipper_"
-#define GXPLUGIN_UI_URI "http://guitarix.sourceforge.net/plugins/gx_aclipper_gui"
+#define GXPLUGIN_UI_URI "http://guitarix.sourceforge.net/plugins/gx_aclipper_gui#_aclipper_"
 
 
 typedef enum
@@ -34,7 +34,8 @@ typedef enum
    EFFECTS_INPUT,
    DRIVE, 
    LEVEL, 
-   TONE, 
+   TONE,
+   BYPASS,
 } PortIndex;
 
 #endif //SRC_HEADERS_GXEFFECTS_H_
