@@ -25,7 +25,7 @@
 #include <lv2.h>
 
 #define GXPLUGIN_URI "http://guitarix.sourceforge.net/plugins/gx_fuzzfacefm_"
-#define GXPLUGIN_UI_URI "http://guitarix.sourceforge.net/plugins/gx_fuzzfacefm_gui"
+#define GXPLUGIN_UI_URI "http://guitarix.sourceforge.net/plugins/gx_fuzzfacefm_gui#_fuzzfacefm_"
 
 
 typedef enum
