@@ -30,8 +30,9 @@
  */
 namespace livelooper {
 
+#define TAPESIZE 4194304 // 2097152 
 
-class LiveLooper: public PluginLV2 {
+class LiveLooper {
 private:
 	int fSamplingFreq;
 	float 	fgain;
@@ -158,6 +159,17 @@ private:
 	float 	*fplay4_;
 	float 	*fplayh4_;
 	float 	*gain4_;
+	float 	*back1_;
+	float 	*back2_;
+	float 	*back3_;
+	float 	*back4_;
+	float 	back1;
+	float 	back2;
+	float 	back3;
+	float 	back4;
+	float 	*sync_tapes_;
+	float 	sync_tapes;
+	int 	clear_buffers;
 	bool save1;
 	bool save2;
 	bool save3;
@@ -183,15 +195,16 @@ private:
     void load_array(std::string name);
     void save_to_wave(std::string fname, float *tape, float fSize);
     int load_from_wave(std::string fname, float *tape);
-    void set_p_state();
+    void set_p_state(Glib::ustring preset_file_name, bool save_file);
     
-	static void clear_state_f_static(PluginLV2*);
-	static int activate_static(bool start, PluginLV2*);
-	static void init_static(unsigned int samplingFreq, PluginLV2*);
-	static void compute_static(int count, float *input0, float *output0, PluginLV2*);
-	static void del_instance(PluginLV2 *p);
-	static void connect_static(uint32_t port,void* data, PluginLV2 *p);
 public:
+	static void clear_state_f_static(LiveLooper*);
+	static int activate_static(bool start, LiveLooper*);
+	static void init_static(unsigned int samplingFreq, LiveLooper*);
+	static void compute_static(int count, float *input0, float *output0, LiveLooper*);
+	static void del_instance(LiveLooper *p);
+	static void connect_static(uint32_t port,void* data, LiveLooper *p);
+    static void set_preset_state(Glib::ustring preset_file_name, bool save_file, LiveLooper*);
 	LiveLooper();
 	~LiveLooper();
 };
@@ -199,8 +212,7 @@ public:
 
 
 LiveLooper::LiveLooper()
-	: PluginLV2(),
-	  tape1(0),
+	: tape1(0),
 	  tape2(0),
 	  tape3(0),
 	  tape4(0),
@@ -217,16 +229,7 @@ LiveLooper::LiveLooper()
       loop_dir("/gxloops/"),
       save_p(false),
 	  mem_allocated(false),
-      ready(false) {
-	id = "dubber";
-	name = N_("Dubber");
-	mono_audio = compute_static;
-	stereo_audio = 0;
-	set_samplerate = init_static;
-	activate_plugin = activate_static;
-    connect_ports = connect_static;
-	delete_instance = del_instance;
-}
+      ready(false) { }
 
 LiveLooper::~LiveLooper() {
     activate(false);
@@ -236,28 +239,28 @@ inline void LiveLooper::clear_state_f()
 {
 	for (int i=0; i<2; i++) fRec0[i] = 0;
 	for (int i=0; i<2; i++) iVec0[i] = 0;
-	for (int i=0; i<4194304; i++) tape1[i] = 0;
+	for (int i=0; i<TAPESIZE; i++) tape1[i] = 0;
 	for (int i=0; i<2; i++) RecSize1[i] = 0;
 	for (int i=0; i<2; i++) fRec1[i] = 0;
 	for (int i=0; i<2; i++) fRec2[i] = 0;
 	for (int i=0; i<2; i++) iRec3[i] = 0;
 	for (int i=0; i<2; i++) iRec4[i] = 0;
 	for (int i=0; i<2; i++) iVec2[i] = 0;
-	for (int i=0; i<4194304; i++) tape2[i] = 0;
+	for (int i=0; i<TAPESIZE; i++) tape2[i] = 0;
 	for (int i=0; i<2; i++) RecSize2[i] = 0;
 	for (int i=0; i<2; i++) fRec6[i] = 0;
 	for (int i=0; i<2; i++) fRec7[i] = 0;
 	for (int i=0; i<2; i++) iRec8[i] = 0;
 	for (int i=0; i<2; i++) iRec9[i] = 0;
 	for (int i=0; i<2; i++) iVec4[i] = 0;
-	for (int i=0; i<4194304; i++) tape3[i] = 0;
+	for (int i=0; i<TAPESIZE; i++) tape3[i] = 0;
 	for (int i=0; i<2; i++) RecSize3[i] = 0;
 	for (int i=0; i<2; i++) fRec11[i] = 0;
 	for (int i=0; i<2; i++) fRec12[i] = 0;
 	for (int i=0; i<2; i++) iRec13[i] = 0;
 	for (int i=0; i<2; i++) iRec14[i] = 0;
 	for (int i=0; i<2; i++) iVec6[i] = 0;
-	for (int i=0; i<4194304; i++) tape4[i] = 0;
+	for (int i=0; i<TAPESIZE; i++) tape4[i] = 0;
 	for (int i=0; i<2; i++) RecSize4[i] = 0;
 	for (int i=0; i<2; i++) fRec16[i] = 0;
 	for (int i=0; i<2; i++) fRec17[i] = 0;
@@ -265,9 +268,9 @@ inline void LiveLooper::clear_state_f()
 	for (int i=0; i<2; i++) iRec19[i] = 0;
 }
 
-void LiveLooper::clear_state_f_static(PluginLV2 *p)
+void LiveLooper::clear_state_f_static(LiveLooper *p)
 {
-	static_cast<LiveLooper*>(p)->clear_state_f();
+	(p)->clear_state_f();
 }
 
 inline void LiveLooper::init(unsigned int samplingFreq)
@@ -284,20 +287,21 @@ inline void LiveLooper::init(unsigned int samplingFreq)
 	fConst0 = (1e+01f / float(fmin(192000, fmax(1, fSamplingFreq))));
 	fConst1 = (0 - fConst0);
     fConst2 = (1.0 / float(fmin(192000, fmax(1, fSamplingFreq))));
+    clear_buffers = 0;
 }
 
-void LiveLooper::init_static(unsigned int samplingFreq, PluginLV2 *p)
+void LiveLooper::init_static(unsigned int samplingFreq, LiveLooper *p)
 {
-	static_cast<LiveLooper*>(p)->init(samplingFreq);
+	(p)->init(samplingFreq);
 }
 
 void LiveLooper::mem_alloc()
 {
     try {
-        if (!tape1) tape1 = new float[4194304];
-        if (!tape2) tape2 = new float[4194304];
-        if (!tape3) tape3 = new float[4194304];
-        if (!tape4) tape4 = new float[4194304];
+        if (!tape1) tape1 = new float[TAPESIZE]{};
+        if (!tape2) tape2 = new float[TAPESIZE]{};
+        if (!tape3) tape3 = new float[TAPESIZE]{};
+        if (!tape4) tape4 = new float[TAPESIZE]{};
         } catch(...) {
             fprintf(stderr, "livelooper out of memory");
             return;
@@ -338,16 +342,16 @@ inline void LiveLooper::load_array(std::string name)
     std::string pPath = getenv("HOME");
     pPath += loop_dir;
     RecSize1[1] = load_from_wave(pPath+name+"1.wav", tape1);
-    IOTAR1= RecSize1[1] - int(RecSize1[1]*(100-fclips1)*0.01);
+    IOTAR1= fmax(0.0f,RecSize1[1] - int(RecSize1[1]*(100-fclips1)*0.01));
     
     RecSize2[1] = load_from_wave(pPath+name+"2.wav", tape2);
-    IOTAR2= RecSize2[1] - int(RecSize2[1]*(100-fclips2)*0.01);
+    IOTAR2= fmax(0.0f,RecSize2[1] - int(RecSize2[1]*(100-fclips2)*0.01));
     
     RecSize3[1] = load_from_wave(pPath+name+"3.wav", tape3);
-    IOTAR3= RecSize3[1] - int(RecSize3[1]*(100-fclips3)*0.01);
+    IOTAR3= fmax(0.0f,RecSize3[1] - int(RecSize3[1]*(100-fclips3)*0.01));
     
     RecSize4[1] = load_from_wave(pPath+name+"4.wav", tape4);
-    IOTAR4= RecSize4[1] - int(RecSize4[1]*(100-fclips4)*0.01);
+    IOTAR4= fmax(0.0f,RecSize4[1] - int(RecSize4[1]*(100-fclips4)*0.01));
     
     cur_name = pfreset_name;
 }
@@ -361,7 +365,7 @@ inline void LiveLooper::save_to_wave(std::string fname, float *tape, float fSize
     
     SNDFILE * sf = sf_open(fname.c_str(), SFM_WRITE, &sfinfo);
     if (sf) {
-        size_t lSize = 4194304 - int(fSize/fConst2);
+        size_t lSize = TAPESIZE - int(fSize/fConst2);
         sf_write_float(sf,tape, lSize);
         sf_write_sync(sf);
     }
@@ -412,13 +416,15 @@ int LiveLooper::activate(bool start)
 	return 0;
 }
 
-int LiveLooper::activate_static(bool start, PluginLV2 *p)
+int LiveLooper::activate_static(bool start, LiveLooper *p)
 {
-	return static_cast<LiveLooper*>(p)->activate(start);
+	return (p)->activate(start);
 }
 
-void LiveLooper::set_p_state() {
-    if (!pfreset_name.empty()) {
+void LiveLooper::set_p_state(Glib::ustring preset_file_name, bool save_file) {
+    if (!preset_file_name.empty()) {
+        pfreset_name = preset_file_name;
+        save_p = save_file;
         ready = false;
         activate(true);
         if(save_p) {
@@ -435,10 +441,16 @@ void LiveLooper::set_p_state() {
     }
 }
 
+void LiveLooper::set_preset_state(Glib::ustring preset_file_name, bool save_file,LiveLooper* p) {
+	(p)->set_p_state(preset_file_name, save_file);
+}
+
 void always_inline LiveLooper::compute(int count, float *input0, float *output0)
 {
     if (!ready) {
-        memcpy(output0, input0, count * sizeof(float));
+        if (input0 != output0) {
+            memcpy(output0, input0, count * sizeof(float));
+        }
         return;
     }
 #define fclip1 (*fclip1_)
@@ -483,6 +495,11 @@ void always_inline LiveLooper::compute(int count, float *input0, float *output0)
 #define freset2 (*freset2_)
 #define freset3 (*freset3_)
 #define freset4 (*freset4_)
+#define back1 (*back1_)
+#define back2 (*back2_)
+#define back3 (*back3_)
+#define back4 (*back4_)
+#define sync_tapes (*sync_tapes_)
     // trigger save array on exit
 	if(record1 || freset1) save1 = true;
     if(record2 || freset2) save2 = true;
@@ -507,13 +524,36 @@ void always_inline LiveLooper::compute(int count, float *input0, float *output0)
     if (freset2) {fclip2=100.0;fclips2=0.0;}
     if (freset3) {fclip3=100.0;fclips3=0.0;}
     if (freset4) {fclip4=100.0;fclips4=0.0;}
+    if(sync_tapes || clear_buffers) {
+        if (clear_buffers ==3) {
+            memset(tape4,0,TAPESIZE*sizeof(float)); clear_buffers =0;
+            rectime3 = TAPESIZE*fConst2;
+        }
+        if (clear_buffers ==2) {
+            memset(tape3,0,TAPESIZE*sizeof(float)); clear_buffers =3;
+            rectime2 = TAPESIZE*fConst2;
+        }
+        if (clear_buffers ==1) {
+            memset(tape2,0,TAPESIZE*sizeof(float)); clear_buffers =2;
+            rectime1 = TAPESIZE*fConst2;
+        }
+        if (freset1) {
+            memset(tape1,0,TAPESIZE*sizeof(float)); clear_buffers =1;
+            rectime0 = TAPESIZE*fConst2;
+        }
+    } 
+    if (!clear_buffers) {
+        if (freset1) {memset(tape1,0,TAPESIZE*sizeof(float));}
+        if (freset2) {memset(tape2,0,TAPESIZE*sizeof(float));}
+        if (freset3) {memset(tape3,0,TAPESIZE*sizeof(float));}
+        if (freset4) {memset(tape4,0,TAPESIZE*sizeof(float));}
+    }
     // switch off freset button when buffer is empty 
-    freset1     = (rectime0 < 4194304*fConst2)? freset1 : 0.0;
-	freset2     = (rectime1 < 4194304*fConst2)? freset2 : 0.0;
-	freset3     = (rectime2 < 4194304*fConst2)? freset3 : 0.0;
-	freset4     = (rectime3 < 4194304*fConst2)? freset4 : 0.0;
+    freset1     = (rectime0 < TAPESIZE*fConst2)? freset1 : 0.0;
+	freset2     = (rectime1 < TAPESIZE*fConst2)? freset2 : 0.0;
+	freset3     = (rectime2 < TAPESIZE*fConst2)? freset3 : 0.0;
+	freset4     = (rectime3 < TAPESIZE*fConst2)? freset4 : 0.0;
     // set fplay head position
-    
     float ph1      = RecSize1[0] ? 1.0/(RecSize1[0] * 0.001) : 0.0;
     fplayh1 = (1-iVec0[0]) * fmin(1000,fmax(0,float(IOTAR1*ph1)));
     float ph2      = RecSize2[0] ? 1.0/(RecSize2[0] * 0.001) : 0.0;
@@ -530,6 +570,28 @@ void always_inline LiveLooper::compute(int count, float *input0, float *output0)
     // engine var settings
 	float 	fSlow0 = (0.0010000000000000009f * powf(10,(0.05f * fgain)));
 	float 	fSlow1 = gain_out;
+    float   iClip1  = fclip1*0.01;
+	float   iClip2  = fclip2*0.01;
+	float   iClip3  = fclip3*0.01;
+	float   iClip4  = fclip4*0.01;
+	float   iClips1  = (100-fclips1)*0.01;
+	float   iClips2  = (100-fclips2)*0.01;
+	float   iClips3  = (100-fclips3)*0.01;
+	float   iClips4  = (100-fclips4)*0.01;
+    if(sync_tapes) {
+        back4 = back3 = back2 = back1;
+        fplayh4 = fplayh3 = fplayh2 = fplayh1;
+        IOTAR4 = IOTAR3 =IOTAR2 = IOTAR1;
+        IOTA4 = IOTA3 =IOTA2 = IOTA1;
+        fplay4 = fplay3 = fplay2 = fplay1;
+        rfplay4 = rfplay3 = rfplay2 = rfplay1;
+        iClip4 = iClip3 = iClip2 = iClip1;
+        iClips4 = iClips3 = iClips2 = iClips1;
+        RecSize4[0] = RecSize3[0] = RecSize2[0] = RecSize1[0];
+        RecSize4[1] = RecSize3[1] = RecSize2[1] = RecSize1[1];
+        speed4 = speed3 = speed2 = speed1;
+        rectime3 = rectime2 = rectime1 = rectime0;
+    }
     int 	iSlow3 = int(record1);
 	int 	iSlow4 = int((1 - freset1));
 	float 	fSlow5 = (((1 - iSlow3) * gain1) * (fplay1+rfplay1));
@@ -543,33 +605,26 @@ void always_inline LiveLooper::compute(int count, float *input0, float *output0)
 	int 	iSlow13 = int((1 - freset4));
 	float 	fSlow14 = (((1 - iSlow12) * gain4) * (fplay4+rfplay4));
 	float 	fSlow15 = (0.0001f * fSlow1);
-    float   iClip1  = fclip1*0.01;
-	float   iClip2  = fclip2*0.01;
-	float   iClip3  = fclip3*0.01;
-	float   iClip4  = fclip4*0.01;
-	float   iClips1  = (100-fclips1)*0.01;
-	float   iClips2  = (100-fclips2)*0.01;
-	float   iClips3  = (100-fclips3)*0.01;
-	float   iClips4  = (100-fclips4)*0.01;
     // run loop
 	for (int i=0; i<count; i++) {
 		fRec0[0] = (fSlow0 + (0.999f * fRec0[1]));
 		float fTemp0 = ((float)input0[i] * fRec0[0]);
 		iVec0[0] = iSlow3;
 		float fTemp1 = (iSlow3 * fTemp0);
-        RecSize1[0] = fmin(4194304, (int)(iSlow4 * (((iSlow3 - iVec0[1]) <= 0) * (iSlow3 + RecSize1[1]))));
-		int iTemp2 = (4194304 - RecSize1[0]);
+        RecSize1[0] = fmin(TAPESIZE, (int)(iSlow4 * (((iSlow3 - iVec0[1]) <= 0) * (iSlow3 + RecSize1[1]))));
+		int iTemp2 = (TAPESIZE - RecSize1[0]);
 		rectime0 = iTemp2*fConst2;
-        int iTemp3 = fmin(4194304-1, (int)(4194304 - iTemp2));
-		if (iSlow3 == 1) {
+        int iTemp3 = fmin(TAPESIZE-1, (int)(TAPESIZE - iTemp2));
+		if ((sync_tapes && (rfplay1 ||fplay1)) || iSlow3 == 1) {
         IOTA1 = IOTA1>int(iTemp3*iClip1)? iTemp3 - int(iTemp3*iClips1):IOTA1+1;
-		tape1[IOTA1] = fTemp1;
+		if (iSlow3 == 1) tape1[IOTA1] = fTemp1;
         }
         if (rfplay1) {
         IOTAR1 = IOTAR1-speed1< (iTemp3 - int(iTemp3*iClips1))? int(iTemp3*iClip1):(IOTAR1-speed1)-1;
         } else if (fplay1) {
         IOTAR1 = IOTAR1+speed1>int(iTemp3*iClip1)? iTemp3 - int(iTemp3*iClips1):(IOTAR1+speed1)+1;
-        } 
+        }
+        if(back1) IOTAR1 = iTemp3 - int(iTemp3*iClips1);
 		
         float fTemp4 = ((int((fRec1[1] != 0.0f)))?((int(((fRec2[1] > 0.0f) & (fRec2[1] < 1.0f))))?fRec1[1]:0):((int(((fRec2[1] == 0.0f) & (iTemp3 != iRec3[1]))))?fConst0:((int(((fRec2[1] == 1.0f) & (iTemp3 != iRec4[1]))))?fConst1:0)));
 		fRec1[0] = fTemp4;
@@ -578,20 +633,21 @@ void always_inline LiveLooper::compute(int count, float *input0, float *output0)
 		iRec4[0] = ((int(((fRec2[1] <= 0.0f) & (iRec3[1] != iTemp3))))?iTemp3:iRec4[1]);
 		iVec2[0] = iSlow6;
 		float fTemp5 = (iSlow6 * fTemp0);
-		RecSize2[0] = fmin(4194304, (int)(iSlow7 * (((iSlow6 - iVec2[1]) <= 0) * (iSlow6 + RecSize2[1]))));
-		int iTemp6 = (4194304 - RecSize2[0]);
+		if (!sync_tapes) RecSize2[0] = fmin(TAPESIZE, (int)(iSlow7 * (((iSlow6 - iVec2[1]) <= 0) * (iSlow6 + RecSize2[1]))));
+		int iTemp6 = (TAPESIZE - RecSize2[0]);
 		rectime1 = iTemp6*fConst2;
-		int iTemp7 = fmin(4194304-1, (int)(4194304 - iTemp6));
-		if (iSlow6 == 1) {
+		int iTemp7 = fmin(TAPESIZE-1, (int)(TAPESIZE - iTemp6));
+		if ((sync_tapes && (rfplay1 ||fplay1)) || iSlow6 == 1) {
 		IOTA2 = IOTA2>int(iTemp7*iClip2)? iTemp7 - int(iTemp7*iClips2):IOTA2+1;
-		tape2[IOTA2] = fTemp5;
+		if (iSlow6 == 1) tape2[IOTA2] = fTemp5;
         }
 		if (rfplay2) {
         IOTAR2 = IOTAR2-speed2< (iTemp7 - int(iTemp7*iClips2))? int(iTemp7*iClip2):(IOTAR2-speed2)-1;
         } else if (fplay2) {
         IOTAR2 = IOTAR2+speed2>int(iTemp7*iClip2)? iTemp7 - int(iTemp7*iClips2):(IOTAR2+speed2)+1;
         }
-        
+        if(back2) IOTAR2 = iTemp7 - int(iTemp7*iClips2);
+       
         float fTemp8 = ((int((fRec6[1] != 0.0f)))?((int(((fRec7[1] > 0.0f) & (fRec7[1] < 1.0f))))?fRec6[1]:0):((int(((fRec7[1] == 0.0f) & (iTemp7 != iRec8[1]))))?fConst0:((int(((fRec7[1] == 1.0f) & (iTemp7 != iRec9[1]))))?fConst1:0)));
 		fRec6[0] = fTemp8;
 		fRec7[0] = fmax(0.0f, fmin(1.0f, (fRec7[1] + fTemp8)));
@@ -599,19 +655,21 @@ void always_inline LiveLooper::compute(int count, float *input0, float *output0)
 		iRec9[0] = ((int(((fRec7[1] <= 0.0f) & (iRec8[1] != iTemp7))))?iTemp7:iRec9[1]);
 		iVec4[0] = iSlow9;
 		float fTemp9 = (iSlow9 * fTemp0);
-		RecSize3[0] = fmin(4194304, (int)(iSlow10 * (((iSlow9 - iVec4[1]) <= 0) * (iSlow9 + RecSize3[1]))));
-		int iTemp10 = (4194304 - RecSize3[0]);
+		if (!sync_tapes) RecSize3[0] = fmin(TAPESIZE, (int)(iSlow10 * (((iSlow9 - iVec4[1]) <= 0) * (iSlow9 + RecSize3[1]))));
+		int iTemp10 = (TAPESIZE - RecSize3[0]);
 		rectime2 = iTemp10*fConst2;
-		int iTemp11 = fmin(4194304-1, (int)(4194304 - iTemp10));
-		if (iSlow9 == 1) {
+		int iTemp11 = fmin(TAPESIZE-1, (int)(TAPESIZE - iTemp10));
+		if ((sync_tapes && (rfplay1 ||fplay1)) || iSlow9 == 1) {
 		IOTA3 = IOTA3>int(iTemp11*iClip3)? iTemp11 - int(iTemp11*iClips3):IOTA3+1;
-		tape3[IOTA3] = fTemp9;
+		if (iSlow9 == 1) tape3[IOTA3] = fTemp9;
         }
 		if (rfplay3) {
         IOTAR3 = IOTAR3-speed3< (iTemp11 - int(iTemp11*iClips3))? int(iTemp11*iClip3):(IOTAR3-speed3)-1;
         } else if (fplay3) {
         IOTAR3 = IOTAR3+speed3>int(iTemp11*iClip3)? iTemp11 - int(iTemp11*iClips3):(IOTAR3+speed3)+1;
         }
+        if(back3) IOTAR3 = iTemp11 - int(iTemp11*iClips3);
+
         float fTemp12 = ((int((fRec11[1] != 0.0f)))?((int(((fRec12[1] > 0.0f) & (fRec12[1] < 1.0f))))?fRec11[1]:0):((int(((fRec12[1] == 0.0f) & (iTemp11 != iRec13[1]))))?fConst0:((int(((fRec12[1] == 1.0f) & (iTemp11 != iRec14[1]))))?fConst1:0)));
 		fRec11[0] = fTemp12;
 		fRec12[0] = fmax(0.0f, fmin(1.0f, (fRec12[1] + fTemp12)));
@@ -619,25 +677,31 @@ void always_inline LiveLooper::compute(int count, float *input0, float *output0)
 		iRec14[0] = ((int(((fRec12[1] <= 0.0f) & (iRec13[1] != iTemp11))))?iTemp11:iRec14[1]);
 		iVec6[0] = iSlow12;
 		float fTemp13 = (iSlow12 * fTemp0);
-		RecSize4[0] = fmin(4194304, (int)(iSlow13 * (((iSlow12 - iVec6[1]) <= 0) * (iSlow12 + RecSize4[1]))));
-		int iTemp14 = (4194304 - RecSize4[0]);
+		if (!sync_tapes) RecSize4[0] = fmin(TAPESIZE, (int)(iSlow13 * (((iSlow12 - iVec6[1]) <= 0) * (iSlow12 + RecSize4[1]))));
+		int iTemp14 = (TAPESIZE - RecSize4[0]);
 		rectime3 = iTemp14*fConst2;
-		int iTemp15 = fmin(4194304-1, (int)(4194304 - iTemp14));
-		if (iSlow12 == 1) {
+		int iTemp15 = fmin(TAPESIZE-1, (int)(TAPESIZE - iTemp14));
+		if ((sync_tapes && (rfplay1 ||fplay1)) || iSlow12 == 1) {
 		IOTA4 = IOTA4>int(iTemp15*iClip4)? iTemp15 - int(iTemp15*iClips4):IOTA4+1;
-		tape4[IOTA4] = fTemp13;
+		if (iSlow12 == 1) tape4[IOTA4] = fTemp13;
         }
 		if (rfplay4) {
         IOTAR4 = IOTAR4-speed4< (iTemp15 - int(iTemp15*iClips4))? int(iTemp15*iClip4):(IOTAR4-speed4)-1;
         } else if (fplay4) {
         IOTAR4 = IOTAR4+speed4>int(iTemp15*iClip4)? iTemp15 - int(iTemp15*iClips4):(IOTAR4+speed4)+1;
         }
+        if(back4) IOTAR4 = iTemp15 - int(iTemp15*iClips4);
+
         float fTemp16 = ((int((fRec16[1] != 0.0f)))?((int(((fRec17[1] > 0.0f) & (fRec17[1] < 1.0f))))?fRec16[1]:0):((int(((fRec17[1] == 0.0f) & (iTemp15 != iRec18[1]))))?fConst0:((int(((fRec17[1] == 1.0f) & (iTemp15 != iRec19[1]))))?fConst1:0)));
 		fRec16[0] = fTemp16;
 		fRec17[0] = fmax(0.0f, fmin(1.0f, (fRec17[1] + fTemp16)));
 		iRec18[0] = ((int(((fRec17[1] >= 1.0f) & (iRec19[1] != iTemp15))))?iTemp15:iRec18[1]);
 		iRec19[0] = ((int(((fRec17[1] <= 0.0f) & (iRec18[1] != iTemp15))))?iTemp15:iRec19[1]);
-		output0[i] = (float)((fSlow15 * ((fSlow14 * ((fRec17[0] * tape4[int(IOTAR4)]) + ((1.0f - fRec17[0]) * tape4[int(IOTAR4)]))) + ((fSlow11 * ((fRec12[0] * tape3[int(IOTAR3)]) + ((1.0f - fRec12[0]) * tape3[int(IOTAR3)]))) + ((fSlow8 * ((fRec7[0] * tape2[int(IOTAR2)]) + ((1.0f - fRec7[0]) * tape2[int(IOTAR2)]))) + (fSlow5 * ((fRec2[0] * tape1[int(IOTAR1)]) + ((1.0f - fRec2[0]) * tape1[int(IOTAR1)]))))))) + (fTemp0));
+        float fTemp17 = (fSlow14 * ((fRec17[0] * tape4[int(IOTAR4)]) + ((1.0f - fRec17[0]) * tape4[int(IOTAR4)])));
+        float fTemp18 = (fSlow11 * ((fRec12[0] * tape3[int(IOTAR3)]) + ((1.0f - fRec12[0]) * tape3[int(IOTAR3)])));
+        float fTemp19 = (fSlow8 * ((fRec7[0] * tape2[int(IOTAR2)]) + ((1.0f - fRec7[0]) * tape2[int(IOTAR2)])));
+        float fTemp20 = (fSlow5 * ((fRec2[0] * tape1[int(IOTAR1)]) + ((1.0f - fRec2[0]) * tape1[int(IOTAR1)])));
+		output0[i] = (float)((fSlow15 * (fTemp17 + (fTemp18 + (fTemp19 + fTemp20)))) + (fTemp0));
 		// post processing
 		iRec19[1] = iRec19[0];
 		iRec18[1] = iRec18[0];
@@ -703,11 +767,15 @@ void always_inline LiveLooper::compute(int count, float *input0, float *output0)
 #undef freset2
 #undef freset3
 #undef freset4
+#undef back1
+#undef back2
+#undef back3
+#undef back4
 }
 
-void __rt_func LiveLooper::compute_static(int count, float *input0, float *output0, PluginLV2 *p)
+void __rt_func LiveLooper::compute_static(int count, float *input0, float *output0, LiveLooper *p)
 {
-	static_cast<LiveLooper*>(p)->compute(count, input0, output0);
+	(p)->compute(count, input0, output0);
 }
 
 void LiveLooper::connect(uint32_t port,void* data)
@@ -840,23 +908,38 @@ void LiveLooper::connect(uint32_t port,void* data)
 	case reset4: 
 		freset4_ = (float*)data; // ,  0.0, 0.0, 1.0, 1.0
 		break;
+	case rback1: 
+		back1_ = (float*)data; // ,  0.0, 0.0, 1.0, 1.0
+		break;
+	case rback2: 
+		back2_ = (float*)data; // ,  0.0, 0.0, 1.0, 1.0
+		break;
+	case rback3: 
+		back3_ = (float*)data; // ,  0.0, 0.0, 1.0, 1.0
+		break;
+	case rback4: 
+		back4_ = (float*)data; // ,  0.0, 0.0, 1.0, 1.0
+		break;
+	case synct: 
+		sync_tapes_ = (float*)data; // ,  0.0, 0.0, 1.0, 1.0
+		break;
 	default:
 		break;
 	}
 }
 
-void LiveLooper::connect_static(uint32_t port,void* data, PluginLV2 *p)
+void LiveLooper::connect_static(uint32_t port,void* data, LiveLooper *p)
 {
-	static_cast<LiveLooper*>(p)->connect(port, data);
+	(p)->connect(port, data);
 }
 
-PluginLV2 *plugin() {
+LiveLooper *plugin() {
 	return new LiveLooper();
 }
 
-void LiveLooper::del_instance(PluginLV2 *p)
+void LiveLooper::del_instance(LiveLooper *p)
 {
-	delete static_cast<LiveLooper*>(p);
+	delete (p);
 }
 
 /*
