@@ -30,15 +30,16 @@
 
 typedef enum
 {
-	INPUTGAIN,
+	INPUT,
 	SWELL,
 	SUSTAIN,
-	OUTPUTGAIN,
+	OUTPUT,
+	BPM,
 	HEAD1,
 	HEAD2,
 	HEAD3,
   	EFFECTS_OUTPUT,
-  	EFFECTS_INPUT,
+  	EFFECTS_INPUT
 } PortIndex;
 
 #endif //SRC_HEADERS_GXEFFECTS_H_
