@@ -216,6 +216,12 @@ static const char* ampstack_groups[] = {
     0
 };
 
+static const char* poweramp_groups[] = {
+    ".poweramp.Pregain",  N_("Gain"),
+    ".poweramp.Gain",  N_("Out"),
+    0
+};
+
 static plugindef_creator builtin_poweramp_plugins[] = {
     gx_poweramps::champ::plugin,
     gx_poweramps::epiphone::plugin,
@@ -273,7 +279,7 @@ GxEngine::GxEngine(const string& plugin_dir, ParameterGroups& groups, const gx_s
 	  "tube.select", _("select"), 0, ampstack_groups),
       poweramps(
 	  *this, "poweramp", _("PowerAmp"), N_("Guitar Effects"), builtin_poweramp_plugins,
-	  "poweramp.mode", _("select"),load_poweramp_ui , 0, PGN_POST_PRE),
+	  "poweramp.mode", _("select"),load_poweramp_ui , poweramp_groups, PGN_POST_PRE),
       // internal audio modules
       noisegate(),
       monomute(),
@@ -345,6 +351,7 @@ void GxEngine::load_static_plugins() {
 
     // * mono amp input position *
 
+    pl.add(dcblocker::plugin(),                   PLUGIN_POS_START, PGN_MODE_NORMAL|PGN_MODE_BYPASS|PGN_MODE_MUTE);
     pl.add(&tuner.plugin,                         PLUGIN_POS_START, PGN_PRE|PGN_MODE_NORMAL|PGN_MODE_BYPASS|PGN_MODE_MUTE);
     pl.add(&noisegate.inputlevel,                 PLUGIN_POS_START, PGN_GUI|PGN_FIXED_GUI|PGN_PRE);
     pl.add(gx_effects::noise_shaper::plugin(),    PLUGIN_POS_START, PGN_GUI|PGN_FIXED_GUI|PGN_PRE);
@@ -372,6 +379,7 @@ void GxEngine::load_static_plugins() {
     pl.add(gx_effects::gx_outputlevel::plugin(),  PLUGIN_POS_END);
     pl.add(balance::plugin(),                     PLUGIN_POS_END, PGN_MODE_BYPASS);
     pl.add(&stereomute,                           PLUGIN_POS_END, PGN_MODE_MUTE);
+    pl.add(hardlim::plugin(),                     PLUGIN_POS_END, PGN_MODE_NORMAL);
     pl.add(&drumout.outputdrum,                   PLUGIN_POS_END, PGN_MODE_NORMAL);
     pl.add(&directout,                            PLUGIN_POS_END, PGN_MODE_NORMAL);
     pl.add(&maxlevel,                             PLUGIN_POS_END, PGN_MODE_NORMAL|PGN_MODE_BYPASS);
