@@ -154,5 +154,7 @@ namespace gx_poweramps {
 
 }
 #include "faust/balance.cc"
+#include "faust/dcblocker.cc"
+#include "faust/hardlim.cc"
 
 }
