@@ -22,12 +22,33 @@
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 #include <sys/ioctl.h>
+#include <sys/types.h>
+#include <sys/socket.h>
 #ifdef HAVE_BLUEZ
 #include <bluetooth/bluetooth.h>
 #include <bluetooth/rfcomm.h>
 #endif
 
+#if !defined(__APPLE__) && !defined(__FreeBSD__)
+#include <malloc.h>
+
+void set_memory_allocation() {
+    if (mlockall(MCL_CURRENT | MCL_FUTURE))
+        gx_print_error("system init", "mlockall failed:");
+#if defined(M_TRIM_THRESHOLD) && defined(M_MMAP_MAX)
+    mallopt(M_TRIM_THRESHOLD, -1);
+    mallopt(M_MMAP_MAX, 0);
+#endif
+}
+
+#else
+
+void set_memory_allocation() {}
+
+#endif
+
 void lock_rt_memory() {
+#ifndef __APPLE__
     extern char __rt_text__start[], __rt_text__end[];
     extern char __rt_data__start[], __rt_data__end[];
     struct {
@@ -37,15 +58,47 @@ void lock_rt_memory() {
 	{ __rt_text__start, __rt_text__end - __rt_text__start },
 	{ __rt_data__start, __rt_data__end - __rt_data__start },
     };
+    long int total_size = 0;
+    set_memory_allocation();
     for (unsigned int i = 0; i < sizeof(regions)/sizeof(regions[0]); i++) {
-	if (mlock(regions[i].start, regions[i].len) != 0) {
-	    gx_print_error(
-		"system init",
-		boost::format(_("failed to lock memory: %1%")) % strerror(errno));
-	}
+        total_size +=regions[i].len;
+        if (mlock(regions[i].start, regions[i].len) != 0) {
+            gx_print_error(
+            "system init",
+            boost::format(_("failed to lock memory: %1%")) % strerror(errno));
+        }
     }
+#ifndef NDEBUG
+    fprintf(stderr,"mlock %ld bytes\n",total_size);
+#endif
+#endif
 }
 
+void unlock_rt_memory() {
+#ifndef __APPLE__    
+    extern char __rt_text__start[], __rt_text__end[];
+    extern char __rt_data__start[], __rt_data__end[];
+    struct {
+    char *start;
+    long len;
+    } regions[] = {
+    { __rt_text__start, __rt_text__end - __rt_text__start },
+    { __rt_data__start, __rt_data__end - __rt_data__start },
+    };
+    long int total_size = 0;
+    for (unsigned int i = 0; i < sizeof(regions)/sizeof(regions[0]); i++) {
+        total_size +=regions[i].len;
+        if (munlock(regions[i].start, regions[i].len) != 0) {
+            gx_print_error(
+            "system init",
+            boost::format(_("failed to unlock memory: %1%")) % strerror(errno));      
+        }
+    }
+#ifndef NDEBUG
+    fprintf(stderr,"munlock %ld bytes\n",total_size);
+#endif  
+#endif
+}
 
 namespace gx_engine {
 
@@ -56,6 +109,27 @@ GxMachineBase::GxMachineBase()
 GxMachineBase::~GxMachineBase() {
 }
 
+void GxMachineBase::set_update_parameter(void *control, const string& id, bool on) {
+    if (on) {
+	update_map[id].insert(control);
+	if (!update_timeout.connected()) {
+	    update_timeout = Glib::signal_timeout().connect(
+		sigc::mem_fun(*this, &GxMachineBase::update_parameter), 60);
+	}
+    } else {
+	output_listen_map::iterator i = update_map.find(id);
+	if (i != update_map.end()) {
+	    i->second.erase(control);
+	    if (i->second.empty()) {
+		update_map.erase(id);
+	    }
+	}
+	if (update_map.empty()) {
+	    update_timeout.disconnect();
+	}
+    }
+}
+
 
 /****************************************************************
  ** GxMachine
@@ -85,6 +159,23 @@ void on_engine_bypass_changed(bool s, GxEngine& engine) {
     }
 }
 
+void GxMachine::process_cmdline_bank_preset() {
+    Glib::ustring sbank = options.get_setbank();
+    if (sbank.empty()) {
+        return;
+    }
+    sbank = sbank.lowercase();
+    if (sbank.size() < 2) {
+        return;
+    }
+    Glib::ustring bank = settings.banks.get_name(
+        KeySwitcher::key_offset_to_bank_idx(sbank[0]-'a', bank_size()));
+    int i = (sbank[1] == ':' ? 2 : 1);
+    Glib::ustring preset = get_bank_file(bank)->get_name(
+        KeySwitcher::key_offset_to_idx(std::stoi(sbank.substr(i))));
+    options.set_bank_preset(bank, preset);
+}
+
 GxMachine::GxMachine(gx_system::CmdlineOptions& options_):
     GxMachineBase(),
     options(options_),
@@ -97,8 +188,10 @@ GxMachine::GxMachine(gx_system::CmdlineOptions& options_):
 #ifdef HAVE_AVAHI
     avahi_service(0),
 #endif
-    pmap(engine.get_param()) {
-    engine.set_jack(&jack);
+    pmap(engine.get_param()),
+    switch_bank() {
+    engine.oscilloscope.set_jack(jack);
+    process_cmdline_bank_preset();
 
     /*
     ** setup parameters
@@ -134,10 +227,21 @@ GxMachine::GxMachine(gx_system::CmdlineOptions& options_):
     pmap.reg_par_non_preset("ui.tuner_reference_pitch", "?Tuner Reference Pitch", 0, 440, 225, 453, 0.1);
     //pmap.reg_par("racktuner.scale_lim", "Limit", &scale_lim, 3.0, 1.0, 10.0, 1.0); FIXME add in detail view?
 
+    static const value_pair midi_channels[] = {{"--"},{"1"},{"2"},{"3"},{"4"},{"5"},{"6"},{"7"},{"8"},{"9"},{"10"},
+        {"11"},{"12"},{"13"},{"14"},{"15"},{"16"}, {0}};
+    EnumParameter* ep = pmap.reg_non_midi_enum_par("system.midi_channel", "Midichannel", midi_channels, (int*)0, false, 0);
+    ep->signal_changed_int().connect(sigc::mem_fun(this, &GxMachineBase::set_midi_channel));
+
     pmap.reg_par("ui.live_play_switcher", "Liveplay preset mode" , (bool*)0, false, false)->setSavable(false);
-    pmap.reg_par("ui.racktuner", N_("Tuner on/off"), (bool*)0, false, false);
+    BoolParameter& up = pmap.reg_par("ui.racktuner", N_("Tuner on/off"), (bool*)0, false, false)->getBool();
+    up.signal_changed().connect(
+        sigc::mem_fun(this, &GxMachineBase::tuner_switch));
     pmap.reg_non_midi_par("system.show_tuner", (bool*)0, false);
     pmap.reg_non_midi_par("system.stick_tuner", (bool*)0, false);
+    BoolParameter& mp = pmap.reg_par("system.midiout_tuner", "Tuner midi output",(bool*)0, false, false)->getBool();
+    mp.signal_changed().connect(
+        sigc::mem_fun(this, &GxMachineBase::tuner_used_by_midi));
+    
     pmap.reg_non_midi_par("system.midi_in_preset", (bool*)0, false, false);
     pmap.reg_par_non_preset("ui.liveplay_brightness", "?liveplay_brightness", 0, 1.0, 0.5, 1.0, 0.01);
     pmap.reg_par_non_preset("ui.liveplay_background", "?liveplay_background", 0, 0.8, 0.0, 1.0, 0.01);
@@ -158,10 +262,11 @@ GxMachine::GxMachine(gx_system::CmdlineOptions& options_):
     pb.signal_changed().connect(
 	sigc::bind(sigc::ptr_fun(on_engine_bypass_changed), sigc::ref(engine)));
     pmap.reg_non_midi_par("ui.mp_s_h", (bool*)0, false);
+    pmap.reg_non_midi_par("ui.all_s_h", (bool*)0, false);
     BoolParameter& ip = pmap.reg_par(
       "engine.insert", N_("switch insert ports on/off"), (bool*)0, false, false)->getBool();
     ip.signal_changed().connect(
-	sigc::mem_fun(this, &GxMachine::set_jack_insert));
+	sigc::mem_fun(this, &GxMachineBase::set_jack_insert));
 
     gx_preset::UnitPresetList presetnames;
     plugin_preset_list_load(pluginlist_lookup_plugin("seq")->get_pdef(), presetnames);
@@ -191,18 +296,20 @@ GxMachine::GxMachine(gx_system::CmdlineOptions& options_):
 	sigc::mem_fun(this, &GxMachine::do_bank_change));
     pmap["ui.live_play_switcher"].signal_changed_bool().connect(
 	sigc::mem_fun(this, &GxMachine::edge_toggle_tuner));
-    engine.midiaudiobuffer.signal_jack_load_change().connect(
-	sigc::mem_fun(this, &GxMachine::on_jack_load_change));
     switch_bank = settings.get_current_bank();
     pmap["engine.next_preset"].signal_changed_bool().connect(
 	sigc::mem_fun(this, &GxMachine::process_next_preset_switch));
     pmap["engine.previus_preset"].signal_changed_bool().connect(
 	sigc::mem_fun(this, &GxMachine::process_previus_preset_switch));
-
+    engine.controller_map.signal_trigger_midi_feedback().connect(
+        sigc::mem_fun(this, &GxMachine::midi_feedback));
+    engine.tuner.signal_freq_changed().connect(
+        sigc::mem_fun(this, &GxMachine::on_tuner_freq_changed));
 }
 
 GxMachine::~GxMachine() {
     stop_socket();
+    unlock_rt_memory();
 #ifndef NDEBUG
     if (options.dump_parameter) {
 	pmap.dump("json");
@@ -221,14 +328,6 @@ void GxMachine::insert_param(Glib::ustring group, Glib::ustring name) {
       sigc::bind(sigc::bind(sigc::bind(sigc::mem_fun(this, &GxMachine::plugin_preset_list_set_on_idle), name), false),pluginlist_lookup_plugin("seq")->get_pdef())));
 }
 
-void GxMachine::on_jack_load_change() {
-    gx_engine::MidiAudioBuffer::Load l = engine.midiaudiobuffer.jack_load_status();
-    if (l == gx_engine::MidiAudioBuffer::load_low && !engine.midiaudiobuffer.get_midistat()) {
-	l = gx_engine::MidiAudioBuffer::load_high;
-    }
-    jack_load_change(l);
-}
-
 void GxMachine::edge_toggle_tuner(bool v) {
     if (v) {
 	tuner_switcher.toggle(engine.tuner.used_for_display());
@@ -354,6 +453,10 @@ float GxMachine::get_tuner_freq() {
     return engine.tuner.get_freq();
 }
 
+float GxMachine::get_tuner_note() {
+    return engine.tuner.get_note();
+}
+
 void GxMachine::set_oscilloscope_mul_buffer(int a) {
     engine.oscilloscope.set_mul_buffer(a, jack.get_jack_bs());
 }
@@ -362,54 +465,6 @@ int GxMachine::get_oscilloscope_mul_buffer() {
     return engine.oscilloscope.get_mul_buffer();
 }
 
-const float *GxMachine::get_oscilloscope_buffer() {
-    return engine.oscilloscope.get_buffer();
-}
-
-void GxMachine::clear_oscilloscope_buffer() {
-    engine.oscilloscope.clear_buffer();
-}
-
-bool GxMachine::oscilloscope_plugin_box_visible() {
-    return engine.oscilloscope.plugin.get_box_visible();
-}
-
-sigc::signal<void, int>& GxMachine::signal_oscilloscope_post_pre() {
-    return pmap[engine.oscilloscope.plugin.id_effect_post_pre()].signal_changed_int();
-}
-
-sigc::signal<void, bool>& GxMachine::signal_oscilloscope_visible() {
-    return pmap[engine.oscilloscope.plugin.id_box_visible()].signal_changed_bool();
-}
-
-sigc::signal<int, bool>& GxMachine::signal_oscilloscope_activation() {
-    return engine.oscilloscope.activation;
-}
-
-sigc::signal<void, unsigned int>& GxMachine::signal_oscilloscope_size_change() {
-    return engine.oscilloscope.size_change;
-}
-
-void GxMachine::maxlevel_get(int channels, float *values) {
-    if (sock) {
-	sock->update_maxlevel();
-	for (int i = 0; i < channels; i++) {
-	    values[i] = sock->get_maxlevel(i);
-	}
-    } else {
-	for (int i = 0; i < channels; i++) {
-	    values[i] = engine.maxlevel.get(i);
-	}
-    }
-}
-
-void GxMachine::get_oscilloscope_info(int& load, int& frames, bool& is_rt, jack_nframes_t& bsize) {
-    load = static_cast<int>(round(jack.get_jcpu_load()));
-    frames = jack.get_time_is()/100000;
-    is_rt = jack.get_is_rt();
-    bsize = jack.get_jack_bs();
-}
-
 gx_system::CmdlineOptions& GxMachine::get_options() const {
     return options;
 }
@@ -470,6 +525,20 @@ void GxMachine::tuner_used_for_display(bool on) {
     engine.tuner.used_for_display(on);
 }
 
+void GxMachine::tuner_used_by_midi(bool on) {
+    if ((on && get_parameter("ui.racktuner").getBool().get_value()) || !on) {
+        engine.tuner.used_by_midi(on);
+    }
+}
+
+void GxMachine::tuner_switch(bool on) {
+    if (!on && get_parameter("system.midiout_tuner").getBool().get_value()) {
+        engine.tuner.used_by_midi(false);
+    } else if (on && get_parameter("system.midiout_tuner").getBool().get_value()) {
+        engine.tuner.used_by_midi(true);
+    }
+}
+
 const std::vector<std::string>& GxMachine::get_rack_unit_order(PluginType type) {
     return settings.get_rack_unit_order(type == PLUGIN_TYPE_STEREO);
 }
@@ -482,16 +551,12 @@ void GxMachine::remove_rack_unit(const std::string& unit, PluginType type) {
     if (!settings.remove_rack_unit(unit, type == PLUGIN_TYPE_STEREO)) {
 	return;
     }
-    if (sock) {
-	sock->send_rack_changed(type == PLUGIN_TYPE_STEREO, 0);
-    }
+    settings.signal_rack_unit_order_changed()(type == PLUGIN_TYPE_STEREO);
 }
 
 void GxMachine::insert_rack_unit(const std::string& unit, const std::string& before, PluginType type) {
     settings.insert_rack_unit(unit, before, type == PLUGIN_TYPE_STEREO);
-    if (sock) {
-	sock->send_rack_changed(type == PLUGIN_TYPE_STEREO, 0);
-    }
+    settings.signal_rack_unit_order_changed()(type == PLUGIN_TYPE_STEREO);
 }
 
 // tuner_switcher
@@ -511,6 +576,26 @@ void GxMachine::tuner_switcher_toggle(bool v) {
     tuner_switcher.toggle(v);
 }
 
+
+void GxMachine::on_tuner_freq_changed() {
+#ifdef USE_MIDI_CC_OUT
+    if (get_parameter("system.midiout_tuner").getBool().get_value()) {
+        float fnote = engine.tuner.get_note();
+        if (fnote < 999.0) {
+            //float tet = get_parameter("racktuner.temperament").getFloat().get_value();
+            int note = static_cast<int>(round(fnote));
+            uint8_t midi_note = static_cast<uint8_t>(note+69);
+            uint8_t vel = static_cast<uint8_t>(((fnote - note) * 127) +63);
+
+            msend_midi_cc(0x90, midi_note,vel, 3);
+        } else {
+            msend_midi_cc(0xB0, 123, 0, 3);
+        }
+    }
+#endif
+}
+
+
 // preset
 bool GxMachine::setting_is_preset() {
     return settings.setting_is_preset();
@@ -537,43 +622,26 @@ Glib::ustring GxMachine::get_bank_name(int n) {
     return settings.banks.get_name(n);
 }
 
-int GxMachine::get_bank_num(Glib::ustring num) {
-    Glib::ustring array = "abcdefghijklmnopqrstuvwxyz" ;
-    int i = 0;
-	for(i=0;i<26;i++) {
-		if(num.compare(array.substr(i,1))==0) break;
-	}
-	return bank_size() -i -1;
-}
-
-void GxMachine::msend_midi_cc(int cc, int pgn, int bgn, int num) {
-	jack.send_midi_cc(cc, pgn, bgn, num);
+bool GxMachine::msend_midi_cc(int cc, int pgn, int bgn, int num) {
+	return jack.send_midi_cc(cc, pgn, bgn, num);
 }
 
 void GxMachine::load_preset(gx_system::PresetFileGui *pf, const Glib::ustring& name) {
     int n = get_bank_index(get_current_bank());
     settings.load_preset(pf, name);
 #ifdef USE_MIDI_CC_OUT
+    bool cc_ok = true;
     if (get_bank_index(pf->get_name()) != n) {
-        msend_midi_cc(0xB0, 32, get_bank_index(pf->get_name()),3);
+        cc_ok = msend_midi_cc(0xB0, 32, get_bank_index(pf->get_name()),3);
+    }
+    if (cc_ok) { // don't send program change if bank switch failed
+        msend_midi_cc(0xC0, pf->get_index(name),0,2);
     }
-    msend_midi_cc(0xC0, pf->get_index(name),0,2);
 #endif
 }
 
-void GxMachine::load_online_presets() {
-    settings.load_online_presets();
-}
-
 void GxMachine::loadstate() {
     settings.loadstate();
-	if (!options.get_setbank().empty()) {
-		Glib::ustring sbank = options.get_setbank();
-		int bl = get_bank_num(sbank.substr(0,1).lowercase());
-		int pgm = max(0,atoi(sbank.substr(2,Glib::ustring::npos).raw().c_str())-1);
-		switch_bank = settings.banks.get_name(bl);
-		do_program_change(pgm);
-    }
 }
 
 int GxMachine::bank_size() {
@@ -634,8 +702,8 @@ sigc::signal<void>& GxMachine::signal_presetlist_changed() {
     return settings.signal_presetlist_changed();
 }
 
-gx_system::PresetFileGui* GxMachine::bank_insert_uri(const Glib::ustring& uri, bool move) {
-    gx_system::PresetFile *f = settings.bank_insert_uri(uri, move);
+gx_system::PresetFileGui* GxMachine::bank_insert_uri(const Glib::ustring& uri, bool move, int position) {
+    gx_system::PresetFile *f = settings.bank_insert_uri(uri, move, position);
     if (f) {
 	return f->get_guiwrapper();
     } else {
@@ -679,6 +747,7 @@ void GxMachine::erase_preset(gx_system::PresetFileGui& pf, const Glib::ustring&
 void GxMachine::bank_set_flag(gx_system::PresetFileGui *pf, int flag, bool v) {
     static_cast<gx_system::PresetFile*>(pf)->set_flag(flag, v);
     settings.banks.save();
+    signal_presetlist_changed()();
 }
 
 std::string GxMachine::bank_get_filename(const Glib::ustring& bank) {
@@ -751,6 +820,23 @@ void GxMachine::set_init_values() {
     pmap.set_init_values();
 }
 
+bool GxMachine::update_parameter() {
+    for (output_listen_map::const_iterator i = update_map.cbegin(); i != update_map.cend(); ++i) {
+	Parameter& p = pmap[i->first];
+	if (p.isMaxlevel()) {
+	    float*f = p.getFloat().value;
+	    if (sock) {
+		*f = sock->update_maxlevel(i->first, true);
+		}
+	    p.trigger_changed();
+	    *f = 0.0;
+	} else {
+	    p.trigger_changed();
+	}
+    }
+    return true;
+}
+
 bool GxMachine::parameter_hasId(const char *p) {
     return pmap.hasId(p);
 }
@@ -768,22 +854,70 @@ bool GxMachine::parameter_unit_has_std_values(const PluginDef *pdef) const {
 }
 
 void GxMachine::set_parameter_value(const std::string& id, int value) {
+#ifdef USE_MIDI_CC_OUT
+    if (options.system_midiout) {
+        const gx_engine::MidiController *pctrl;
+        IntParameter& p = get_parameter(id).getInt();
+        if (p.get_value() != value) {
+            int nctl = midi_param2controller(p, &pctrl);
+            if (nctl > -1 && nctl < 128) {
+                int state = int(((float(value) - p.getLowerAsFloat()) /
+                    (p.getUpperAsFloat() - p.getLowerAsFloat())) * 127.0);
+                msend_midi_cc(0xB0, nctl, state, 3);
+                if (pctrl->toggle_behaviour() != Parameter::toggle_type::OnOff) {
+                    engine.controller_map.set_last_midi_control_value(nctl, state);
+                } else {
+                    engine.controller_map.set_last_midi_control_value(nctl, p.get_value()*127);
+                }
+            }
+        }
+    }
+#endif
     pmap[id].getInt().set(value);
 }
 
 void GxMachine::set_parameter_value(const std::string& id, bool value) {
+#ifdef USE_MIDI_CC_OUT
+    if (options.system_midiout) {
+        const gx_engine::MidiController *pctrl;
+        BoolParameter& p = get_parameter(id).getBool();
+        if (p.get_value() != value) {
+            int nctl = midi_param2controller(p, &pctrl);
+            if (nctl > -1 && nctl < 128) {
+                int state = int(value * 127);
+                msend_midi_cc(0xB0, nctl,state, 3);
+                if (pctrl->toggle_behaviour() != Parameter::toggle_type::OnOff) {
+                    engine.controller_map.set_last_midi_control_value(nctl, state);
+                } else {
+                    engine.controller_map.set_last_midi_control_value(nctl, p.get_value()*127);
+                }
+            }
+        }
+    }
+#endif
     pmap[id].getBool().set(value);
 }
 
- //bool GxMachine::ui_f_update(const std::string& id, float value) {
- //    pmap[id].getFloat().set(value);
- //    return false;
- //}
-
 void GxMachine::set_parameter_value(const std::string& id, float value) {
-    // Glib::signal_timeout().connect(
-    //     sigc::bind<const std::string&>(sigc::bind<float>(
-     //    sigc::mem_fun (*this, &GxMachine::ui_f_update),value),id), 20);
+#ifdef USE_MIDI_CC_OUT
+    if (options.system_midiout) {
+        const gx_engine::MidiController *pctrl;
+        FloatParameter& p = get_parameter(id).getFloat();
+        if ( std::fabs(p.get_value() - value) > 0.00001) {
+            int nctl = midi_param2controller(p, &pctrl);
+            if (nctl > -1 && nctl < 128) {
+                int state = int(((float(value) - p.getLowerAsFloat()) /
+                    (p.getUpperAsFloat() - p.getLowerAsFloat())) * 127.0);
+                msend_midi_cc(0xB0, nctl, state, 3);
+               // if (pctrl->toggle_behaviour() != Parameter::toggle_type::OnOff) {
+               //     engine.controller_map.set_last_midi_control_value(nctl, state);
+               // } else {
+               //      engine.controller_map.set_last_midi_control_value(nctl, p.get_value()*127);
+               // }
+            }
+        }
+    }
+#endif
     pmap[id].getFloat().set(value);
 }
 
@@ -869,6 +1003,21 @@ int GxMachine::midi_param2controller(Parameter& param, const MidiController** p)
     return engine.controller_map.param2controller(param, p);
 }
 
+void GxMachine::set_midi_channel(int s) {
+    return engine.controller_map.set_midi_channel(s);
+}
+
+void GxMachine::midi_feedback(int c, int v) {
+#ifdef USE_MIDI_CC_OUT
+    if (options.system_midiout) {
+        // Don't send midi feedback for bpm / jack_transport and not on/off events
+        if ( c !=22 && c != 24 && c < 128) {
+            msend_midi_cc(0xB0, c,v, 3);
+        }
+    }
+#endif
+}
+
 // Convolver
 
 void GxMachine::on_impresp(const std::string& path) {
@@ -937,10 +1086,6 @@ GxMachineRemote::GxMachineRemote(gx_system::CmdlineOptions& options_)
       current_preset(),
       bank_drag_get_counter(),
       bank_drag_get_path(),
-      oscilloscope_activation(),
-      oscilloscope_size_change(),
-      oscilloscope_buffer(0),
-      oscilloscope_buffer_size(0),
       tuner_switcher_display(),
       tuner_switcher_set_state(),
       tuner_switcher_selection_done() {
@@ -950,8 +1095,13 @@ GxMachineRemote::GxMachineRemote(gx_system::CmdlineOptions& options_)
 	create_tcp_socket();
     }
     socket->set_blocking(true);
-    writebuf = new __gnu_cxx::stdio_filebuf<char>(socket->get_fd(), std::ios::out);
-    os = new ostream(writebuf);
+    //writebuf = new __gnu_cxx::stdio_filebuf<char>(socket->get_fd(), std::ios::out);
+    writebuf = new  boost::iostreams::file_descriptor_sink;
+    writebuf->open(socket->get_fd(),boost::iostreams::never_close_handle);
+    
+    // os = new ostream(writebuf);
+    os = new boost::iostreams::stream<boost::iostreams::file_descriptor_sink>(*writebuf);
+    
     jw = new gx_system::JsonWriter(os, false);
 
     START_CALL(parameterlist);
@@ -981,8 +1131,6 @@ GxMachineRemote::GxMachineRemote(gx_system::CmdlineOptions& options_)
     jw->write("presetlist_changed");
     jw->write("logger");
     jw->write("midi");
-    jw->write("oscilloscope");
-    jw->write("jack_load");
     jw->write("param");
     jw->write("plugins_changed");
     jw->write("misc");
@@ -1069,12 +1217,13 @@ void GxMachineRemote::create_bluetooth_socket(const Glib::ustring& bdaddr) {
 void GxMachineRemote::create_tcp_socket() {
     socket = Gio::Socket::create(Gio::SOCKET_FAMILY_IPV4, Gio::SOCKET_TYPE_STREAM, Gio::SOCKET_PROTOCOL_TCP);
     int flag = 1;
-    setsockopt(socket->get_fd(), IPPROTO_TCP, TCP_NODELAY, &flag, sizeof(int));
+    if (setsockopt(socket->get_fd(), IPPROTO_TCP, TCP_NODELAY, &flag, sizeof(int)))
+        gx_print_error("GxMachineRemote","setsockopt(IPPROTO_TCP, TCP_NODELAY) failed");
     typedef std::vector< Glib::RefPtr<Gio::InetAddress> > adr_list;
     adr_list al;
     try {
 	al = Gio::Resolver::get_default()->lookup_by_name(options.get_rpcaddress());
-    } catch (Glib::Error e) {
+    } catch (Glib::Error& e) {
 	gx_print_fatal(_("Remote Connection"), e.what());
     }
     Glib::ustring msg;
@@ -1083,7 +1232,7 @@ void GxMachineRemote::create_tcp_socket() {
 	try {
 	    socket->connect(Gio::InetSocketAddress::create(*i, options.get_rpcport()));
 	    error = false;
-	} catch (Gio::Error e) {
+	} catch (Gio::Error& e) {
 	    msg = e.what();
 	    error = true;
 	}
@@ -1165,6 +1314,10 @@ void GxMachineRemote::parameter_changed(gx_system::JsonStringParser *jp) {
 	SeqParameter* pj = dynamic_cast<SeqParameter*>(&p);
 	pj->readJSON_value(*jp);
 	pj->setJSON_value();
+    } else if (dynamic_cast<OscParameter*>(&p) != 0) {
+	OscParameter* po = dynamic_cast<OscParameter*>(&p);
+	po->readJSON_value(*jp);
+	po->setJSON_value();
     } else {
 	cerr << "change special type parameter " << p.id() << endl;
     }
@@ -1232,18 +1385,6 @@ void GxMachineRemote::handle_notify(gx_system::JsonStringParser *jp) {
 	int value = jp->current_value_int();
 	jp->next(gx_system::JsonParser::end_array);
 	midi_value_changed(ctl, value);
-    } else if (method == "osc_activation") {
-	jp->next(gx_system::JsonParser::value_number);
-	oscilloscope_activation(jp->current_value_int());
-    } else if (method == "osc_size_changed") {
-	jp->next(gx_system::JsonParser::value_number);
-	unsigned int sz = jp->current_value_int();
-	if (oscilloscope_buffer_size != sz) {
-	    delete oscilloscope_buffer;
-	    oscilloscope_buffer = new float[sz];
-	    oscilloscope_buffer_size = sz;
-	}
-	oscilloscope_size_change(sz);
     } else if (method == "show_tuner") {
 	jp->next(gx_system::JsonParser::value_number);
 	tuner_switcher_selection_done(jp->current_value_int());
@@ -1282,9 +1423,6 @@ void GxMachineRemote::handle_notify(gx_system::JsonStringParser *jp) {
 	impresp_list(path, l);
     } else if (method == "plugins_changed") {
 	update_plugins(jp);
-    } else if (method == "jack_load_changed") {
-	jp->next(gx_system::JsonParser::value_number);
-	jack_load_change(static_cast<gx_engine::MidiAudioBuffer::Load>(jp->current_value_int()));
     } else if (method == "server_shutdown") {
 	Gtk::Main::quit();
     } else {
@@ -1319,7 +1457,7 @@ bool GxMachineRemote::socket_input_handler(Glib::IOCondition cond) {
     while (true) {
 	try {
 	    n = socket->receive(buf, sizeof(buf));
-	} catch(Glib::Error e) {
+	} catch(Glib::Error& e) {
 	    delete jp;
 	    socket_error(2);
 	    return false;
@@ -1340,7 +1478,7 @@ bool GxMachineRemote::socket_input_handler(Glib::IOCondition cond) {
 		    jp->next(gx_system::JsonParser::value_string); // "2.0"
 		    jp->next(gx_system::JsonParser::value_key); // "method"
 		    handle_notify(jp);
-		} catch (gx_system::JsonException e) {
+		} catch (gx_system::JsonException& e) {
 		    cerr << "JsonException: " << e.what() << ": '" << jp->get_string() << "'" << endl;
 		    assert(false);
 		}
@@ -1448,7 +1586,7 @@ gx_system::JsonStringParser *GxMachineRemote::receive() {
 	    int n;
 	    try {
 		n = socket->receive(buf, sizeof(buf));
-	    } catch(Glib::Error e) {
+	    } catch(Glib::Error& e) {
 		cerr << "Glib receive error: " << e.what() << endl;
 		return 0;
 	    }
@@ -1625,11 +1763,6 @@ static const std::string next_string(gx_system::JsonParser *jp) {
     return jp->current_value();
 }
 
-int GxMachineRemote::load_remote_ui_static (const UiBuilder& builder, int form) {
-    GxMachineRemote *m = dynamic_cast<GxMachineRemote*>(&static_cast<const gx_gui::UiBuilderImpl*>(&builder)->main.get_machine());
-    return m->load_remote_ui(builder, form);
-}
-
 int GxMachineRemote::load_remote_ui(const UiBuilder& builder, int form) {
     START_CALL(plugin_load_ui);
     jw->write(builder.plugin->id);
@@ -1761,7 +1894,12 @@ int GxMachineRemote::load_remote_ui(const UiBuilder& builder, int form) {
 }
 
 bool GxMachineRemote::load_unit(gx_gui::UiBuilderImpl& builder, PluginDef* pdef) {
-    pdef->load_ui = load_remote_ui_static;
+    pdef->load_ui = [](const UiBuilder& builder, int form) {
+                        GxMachineRemote *m = dynamic_cast<GxMachineRemote*>(
+                            &static_cast<const gx_gui::UiBuilderImpl*>(
+                                &builder)->plugin_dict.get_machine());
+                        return m->load_remote_ui(builder, form);
+                    };
     return builder.load_unit(pdef);
 }
 
@@ -1783,35 +1921,6 @@ int GxMachineRemote::get_oscilloscope_mul_buffer() {
     END_RECEIVE(return 1);
 }
 
-const float *GxMachineRemote::get_oscilloscope_buffer() {
-    return oscilloscope_buffer;
-}
-
-void GxMachineRemote::clear_oscilloscope_buffer() {
-    START_NOTIFY(clear_oscilloscope_buffer);
-    SEND();
-}
-
-bool GxMachineRemote::oscilloscope_plugin_box_visible() {
-    return pluginlist.lookup_plugin("oscilloscope")->get_box_visible();
-}
-
-sigc::signal<void, int>& GxMachineRemote::signal_oscilloscope_post_pre() {
-    return pmap["oscilloscope.pp"].signal_changed_int();
-}
-
-sigc::signal<void, bool>& GxMachineRemote::signal_oscilloscope_visible() {
-    return pmap["ui.oscilloscope"].signal_changed_bool();
-}
-
-sigc::signal<int, bool>& GxMachineRemote::signal_oscilloscope_activation() {
-    return oscilloscope_activation;
-}
-
-sigc::signal<void, unsigned int>& GxMachineRemote::signal_oscilloscope_size_change() {
-    return oscilloscope_size_change;
-}
-
 float GxMachineRemote::get_tuner_freq() {
     START_CALL(get_tuner_freq);
     START_RECEIVE(0);
@@ -1820,51 +1929,12 @@ float GxMachineRemote::get_tuner_freq() {
     END_RECEIVE(return 0);
 }
 
-void GxMachineRemote::maxlevel_get(int channels, float *values) {
-    START_CALL(get_max_output_level);
-    jw->write(channels);
-    START_RECEIVE();
-    jp->next(gx_system::JsonParser::begin_array);
-    for (int i = 0; i < channels; i++) {
-	if (jp->peek() != gx_system::JsonParser::end_array) {
-	    jp->next(gx_system::JsonParser::value_number);
-	    values[i] = jp->current_value_float();
-	} else {
-	    values[i] = 0.0;
-	}
-    }
-    END_RECEIVE();
-}
-
-void GxMachineRemote::get_oscilloscope_info(int& load, int& frames, bool& is_rt, jack_nframes_t& bsize) {
-    START_CALL(get_oscilloscope_info);
-    START_RECEIVE();
-    jp->next(gx_system::JsonParser::begin_array);
+float GxMachineRemote::get_tuner_note() {
+    START_CALL(get_tuner_note);
+    START_RECEIVE(0);
     jp->next(gx_system::JsonParser::value_number);
-    load = jp->current_value_int();
-    jp->next(gx_system::JsonParser::value_number);
-    frames = jp->current_value_int();
-    jp->next(gx_system::JsonParser::value_number);
-    is_rt = jp->current_value_int();
-    jp->next(gx_system::JsonParser::value_number);
-    bsize = jp->current_value_int();
-    jp->next(gx_system::JsonParser::value_number);
-    unsigned int sz = jp->current_value_int();
-    if (oscilloscope_buffer_size != sz) {
-	delete oscilloscope_buffer;
-	oscilloscope_buffer = new float[sz];
-	oscilloscope_buffer_size = sz;
-	oscilloscope_size_change(sz);
-    }
-    jp->next(gx_system::JsonParser::begin_array);
-    float *p = oscilloscope_buffer;
-    while (jp->peek() != gx_system::JsonParser::end_array) {
-	jp->next(gx_system::JsonParser::value_number);
-	*p++ = jp->current_value_float();
-    }
-    jp->next(gx_system::JsonParser::end_array);
-    jp->next(gx_system::JsonParser::end_array);
-    END_RECEIVE();
+    return jp->current_value_float();
+    END_RECEIVE(return 0);
 }
 
 gx_system::CmdlineOptions& GxMachineRemote::get_options() const {
@@ -1900,6 +1970,22 @@ void GxMachineRemote::tuner_used_for_display(bool on) {
     SEND();
 }
 
+void GxMachineRemote::tuner_used_by_midi(bool on) {
+    if ((on && get_parameter("ui.racktuner").getBool().get_value()) || !on) {
+        START_NOTIFY(tuner_used_by_midi);
+        jw->write(on);
+        SEND();
+    }
+}
+
+void GxMachineRemote::tuner_switch(bool on) {
+    if (!on && get_parameter("system.midiout_tuner").getBool().get_value()) {
+        tuner_used_by_midi(false);
+    } else if (on && get_parameter("system.midiout_tuner").getBool().get_value()) {
+        tuner_used_by_midi(true);
+    }
+}
+
 const std::vector<std::string>& GxMachineRemote::get_rack_unit_order(PluginType type) {
     bool stereo = (type == PLUGIN_TYPE_STEREO);
     std::vector<std::string>& l = (stereo ? rack_units.stereo : rack_units.mono);
@@ -1914,7 +2000,7 @@ const std::vector<std::string>& GxMachineRemote::get_rack_unit_order(PluginType
 	    l.push_back(jp->current_value());
 	}
 	jp->next(gx_system::JsonParser::end_array);
-    } catch (gx_system::JsonException e) {
+    } catch (gx_system::JsonException& e) {
 	cerr << "JsonException: " << e.what() << ": '" << jp->get_string() << "'" << endl;
 	assert(false);
     }
@@ -1931,6 +2017,7 @@ void GxMachineRemote::remove_rack_unit(const std::string& unit, PluginType type)
     jw->write(unit);
     jw->write(type == PLUGIN_TYPE_STEREO);
     SEND();
+    rack_units.rack_unit_order_changed(type == PLUGIN_TYPE_STEREO);
 }
 
 void GxMachineRemote::insert_rack_unit(const std::string& unit, const std::string& before, PluginType type) {
@@ -1939,6 +2026,7 @@ void GxMachineRemote::insert_rack_unit(const std::string& unit, const std::strin
     jw->write(before);
     jw->write(type == PLUGIN_TYPE_STEREO);
     SEND();
+    rack_units.rack_unit_order_changed(type == PLUGIN_TYPE_STEREO);
 }
 
 // tuner_switcher
@@ -1966,6 +2054,23 @@ void GxMachineRemote::tuner_switcher_toggle(bool v) {
     SEND();
 }
 
+void GxMachineRemote::on_tuner_freq_changed() {
+#ifdef USE_MIDI_CC_OUT
+    if (get_parameter("system.midiout_tuner").getBool().get_value()) {
+        float fnote = get_tuner_note();
+        if (fnote < 999.0) {
+            int note = static_cast<int>(round(fnote));
+            uint8_t midi_note = static_cast<uint8_t>(note+69);
+            uint8_t vel = static_cast<uint8_t>(((fnote - note) * 127) +63);
+
+            msend_midi_cc(0x90, midi_note,vel, 3);
+        } else {
+            msend_midi_cc(0xB0, 123, 0, 3);
+        }
+    }
+#endif
+}
+
 // preset
 
 void GxMachineRemote::next_preset_switch() {
@@ -2025,13 +2130,14 @@ Glib::ustring GxMachineRemote::get_bank_name(int n) {
     return banks.get_name(n);
 }
 
-void GxMachineRemote::msend_midi_cc(int cc, int pgn, int bgn, int num) {
+bool GxMachineRemote::msend_midi_cc(int cc, int pgn, int bgn, int num) {
 	START_NOTIFY(sendcc);
     jw->write(cc);
     jw->write(pgn);
     jw->write(bgn);
     jw->write(num);
     SEND();
+    return true;
 }
 
 void GxMachineRemote::load_preset(gx_system::PresetFileGui *pf, const Glib::ustring& name) {
@@ -2048,11 +2154,6 @@ void GxMachineRemote::load_preset(gx_system::PresetFileGui *pf, const Glib::ustr
 #endif
 }
 
-void GxMachineRemote::load_online_presets() {
-    START_NOTIFY(set_online_presets);
-    SEND();
-}
-
 void GxMachineRemote::loadstate() {
     /* noop */
 }
@@ -2145,7 +2246,7 @@ sigc::signal<void>& GxMachineRemote::signal_presetlist_changed() {
     return presetlist_changed;
 }
 
-gx_system::PresetFileGui *GxMachineRemote::bank_insert_uri(const Glib::ustring& uri, bool move) {
+gx_system::PresetFileGui *GxMachineRemote::bank_insert_uri(const Glib::ustring& uri, bool move, int position) {
     START_CALL(bank_insert_content);
     jw->write(uri);
     Glib::RefPtr<Gio::File> rem = Gio::File::create_for_uri(uri);
@@ -2153,13 +2254,14 @@ gx_system::PresetFileGui *GxMachineRemote::bank_insert_uri(const Glib::ustring&
     stringstream s;
     s << f.rdbuf();
     jw->write(s.str());
+    jw->write(position);
     START_RECEIVE(0);
     if (jp->peek() != gx_system::JsonParser::begin_object) {
 	return 0;
     }
     gx_system::PresetFile *pf = new gx_system::PresetFile();
     pf->readJSON_remote(*jp);
-    banks.insert(pf);
+    banks.insert(pf, position);
     return pf->get_guiwrapper();
     END_RECEIVE(return 0);
 }
@@ -2495,6 +2597,15 @@ void GxMachineRemote::set_init_values() {
     }
 }
 
+bool GxMachineRemote::update_parameter() {
+    START_NOTIFY(get_updates);
+    for (output_listen_map::const_iterator i = update_map.cbegin(); i != update_map.cend(); ++i) {
+	jw->write(i->first);
+    }
+    SEND();
+    return true;
+}
+
 bool GxMachineRemote::parameter_hasId(const char *p) {
     return pmap.hasId(p);
 }
@@ -2512,14 +2623,70 @@ bool GxMachineRemote::parameter_unit_has_std_values(const PluginDef *pdef) const
 }
 
 void GxMachineRemote::set_parameter_value(const std::string& id, int value) {
+#ifdef USE_MIDI_CC_OUT
+    if (options.system_midiout) {
+        const gx_engine::MidiController *pctrl;
+        IntParameter& p = get_parameter(id).getInt();
+        if (p.get_value() != value) {
+            int nctl = midi_param2controller(p, &pctrl);
+            if (nctl > -1 && nctl < 128) {
+                int state = int(((float(value) - p.getLowerAsFloat()) /
+                    (p.getUpperAsFloat() - p.getLowerAsFloat())) * 127.0);
+                msend_midi_cc(0xB0, nctl, state, 3);
+                if (pctrl->toggle_behaviour() != Parameter::toggle_type::OnOff) {
+                    midi_set_last_controller_value(nctl, state);
+                } else {
+                     midi_set_last_controller_value(nctl, p.get_value()*127);
+                }
+            }
+        }
+    }
+#endif
     pmap[id].getInt().set(value);
 }
 
 void GxMachineRemote::set_parameter_value(const std::string& id, bool value) {
+#ifdef USE_MIDI_CC_OUT
+    if (options.system_midiout) {
+        const gx_engine::MidiController *pctrl;
+        BoolParameter& p = get_parameter(id).getBool();
+        if (p.get_value() != value) {
+            int nctl = midi_param2controller(p, &pctrl);
+            if (nctl > -1 && nctl < 128) {
+                int state = int(value * 127);
+                msend_midi_cc(0xB0, nctl,state, 3);
+                if (pctrl->toggle_behaviour() != Parameter::toggle_type::OnOff) {
+                    midi_set_last_controller_value(nctl, state);
+                } else {
+                     midi_set_last_controller_value(nctl, p.get_value()*127);
+                }
+            }
+        }
+    }
+#endif
     pmap[id].getBool().set(value);
 }
 
 void GxMachineRemote::set_parameter_value(const std::string& id, float value) {
+#ifdef USE_MIDI_CC_OUT
+    if (options.system_midiout) {
+        const gx_engine::MidiController *pctrl;
+        FloatParameter& p = get_parameter(id).getFloat();
+        if ( std::fabs(p.get_value() - value) > 0.00001) {
+            int nctl = midi_param2controller(p, &pctrl);
+            if (nctl > -1 && nctl < 128) {
+                int state = int(((float(value) - p.getLowerAsFloat()) /
+                    (p.getUpperAsFloat() - p.getLowerAsFloat())) * 127.0);
+                msend_midi_cc(0xB0, nctl, state, 3);
+               // if (pctrl->toggle_behaviour() != Parameter::toggle_type::OnOff) {
+               //     midi_set_last_controller_value(nctl, state);
+               // } else {
+               //      midi_set_last_controller_value(nctl, p.get_value()*127);
+               // }
+            }
+        }
+    }
+#endif
     pmap[id].getFloat().set(value);
 }
 
@@ -2680,6 +2847,42 @@ int GxMachineRemote::midi_param2controller(Parameter& param, const MidiControlle
     return midi_controller_map.param2controller(param, p);
 }
 
+void GxMachineRemote::set_midi_channel(int s) {
+    START_NOTIFY(set_midi_channel);
+    jw->write(s);
+    SEND();
+}
+
+int GxMachineRemote::midi_get_last_controller_value(int ctl) {
+    START_CALL(get_last_midi_control_value);
+    jw->write(ctl);
+    START_RECEIVE(false);
+    jp->next(gx_system::JsonParser::begin_object);
+    jp->next(gx_system::JsonParser::value_number);
+    return jp->current_value_int();
+    END_RECEIVE(return false);
+    
+}
+
+void GxMachineRemote::midi_set_last_controller_value(int ctl, int v) {
+    START_NOTIFY(set_last_midi_control_value);
+    jw->write(ctl);
+    jw->write(v);
+    SEND();
+    
+}
+
+void GxMachineRemote::midi_feedback(int c, int v) {
+#ifdef USE_MIDI_CC_OUT
+    if (options.system_midiout) {
+        // Don't send midi feedback for bpm / jack_transport and not on/off events
+        if ( c !=22 && c != 24 && c < 128) {
+            msend_midi_cc(0xB0, c,v, 3);
+        }
+    }
+#endif
+}
+
 // Convolver
 
 void GxMachineRemote::reload_impresp_list(const std::string& path) {
