@@ -154,6 +154,8 @@ declare(supersonic)
 }
 
 declare(balance)
+declare(dcblocker)
+declare(hardlim)
 
 } // namespace gx_engine
 
