@@ -106,11 +106,10 @@ class FileParameter;
 class Parameter: boost::noncopyable {
 public:
     enum ctrl_type { None, Continuous, Switch, Enum };
-    enum toggle_type { OnOff = 0, Constant = 1, _Count = 2 };
+    enum toggle_type { OnOff = 0, Constant = 1, Toggle = 2, Commander = 3, _Count = 4 };
 private:
     virtual bool midi_set(float n, float high, float llimit, float ulimit); //RT
     virtual bool midi_set_bpm(float n, float high, float llimit, float ulimit); //RT
-    virtual void trigger_changed();
     friend class MidiController;
 protected:
     enum value_type { tp_float, tp_int, tp_bool, tp_file, tp_string, tp_special };
@@ -125,6 +124,9 @@ protected:
     bool do_not_save       : 1;
     bool blocked           : 1;
     bool midi_blocked      : 1;
+    bool output            : 1;
+    bool maxlevel          : 1;
+    bool nowarn            : 1;
     bool used              : 1; // debug
 protected:
     void range_warning(float value, float lower, float upper);
@@ -146,10 +148,13 @@ public:
 	do_not_save(false),
 	blocked(false),
 	midi_blocked(false),
+	output(false),
+	maxlevel(false),
         used(false) {}
     Parameter(gx_system::JsonParser& jp);
     virtual ~Parameter();
     virtual void serializeJSON(gx_system::JsonWriter& jw);
+    virtual void trigger_changed();
 
 #ifndef NDEBUG
     bool isUsed() const { return used; }
@@ -170,6 +175,12 @@ public:
     bool isInPreset() const { return save_in_preset; }
     bool isSavable() const { return !do_not_save; }
     void setSavable(bool v) { do_not_save = !v; }
+    bool isOutput() const { return output; }
+    void setOutput(bool v) { output = v; }
+    bool isMaxlevel() const { return maxlevel; }
+    void setMaxlevel(bool v) { maxlevel = v; }
+    bool isNoWarning() const { return nowarn; }
+    void setNoWarning(bool v) { nowarn = v; }
     const string& id() const { return _id; }
     const string& group() const { return _group; }
     string l_group() const { return gettext(_group.c_str()); }
@@ -179,11 +190,11 @@ public:
     void set_desc(const string& desc) { _desc = desc; }
     string l_desc() const { return gettext(_desc.c_str()); }
     void set_log_display() { d_flags |= dtp_log; }
-    bool is_log_display() { return d_flags & dtp_log; }
+    bool is_log_display() const { return d_flags & dtp_log; }
     void set_blocked(bool v) { blocked = v; }
-    bool get_blocked() { return blocked; }
+    bool get_blocked() const { return blocked; }
     void set_midi_blocked(bool v) { midi_blocked = v; }
-    bool get_midi_blocked() { return midi_blocked; }
+    bool get_midi_blocked() const { return midi_blocked; }
     bool operator==(const Parameter& p) const { return &p == this; }
     virtual void stdJSON_value() = 0;
     virtual bool on_off_value() = 0; //RT
@@ -240,11 +251,13 @@ protected:
     sigc::signal<void, float> changed;
     float value_storage;
     friend class ParamRegImpl;
+    friend class GxMachine;
 public:
     bool set(float val) const;
     bool ramp_value(float val);
     float get_value() const { return *value; }
     void convert_from_range(float low, float up);
+    void set_zero() { *value = 0; }
     virtual void stdJSON_value();
     virtual bool on_off_value();
     virtual void writeJSON(gx_system::JsonWriter& jw) const;
@@ -261,7 +274,10 @@ public:
 	       float *v, float sv, float lv, float uv, float tv, bool ctrl, bool no_init):
 	Parameter(id, name, tp_float, ctp, preset, ctrl),
 	value(v ? v : &value_storage), std_value(sv),lower(lv),upper(uv),step(tv) {
-	set(no_init ? *value : sv);
+        assert(lv <= sv && sv <= uv);
+        if (!no_init) {
+            *value = sv;
+        }
     }
 #ifndef NDEBUG
     friend void compare_parameter(const char* title, Parameter* p1,
@@ -303,6 +319,7 @@ protected:
     int lower, upper;
     sigc::signal<void, int> changed;
     int value_storage;
+    friend class ParamRegImpl;
 public:
     bool set(int val) const;
     int get_value() const { return *value; }
@@ -358,6 +375,7 @@ protected:
     bool std_value;
     sigc::signal<void, bool> changed;
     bool value_storage;
+    friend class ParamRegImpl;
 public:
     bool set(bool val) const;
     virtual void stdJSON_value();
@@ -447,6 +465,7 @@ public:
     ParameterV(gx_system::JsonParser& jp);
     virtual void serializeJSON(gx_system::JsonWriter& jw);
     sigc::signal<void, const Glib::ustring&>& signal_changed() { return changed; }
+    Glib::ustring& get_json_value() { return json_value; }
 };
 
 
@@ -553,8 +572,8 @@ class ParamMap: boost::noncopyable {
     void unregister(Parameter *p);
     void unregister(const string& id);
     inline FloatParameter *reg_par(const string& id, const string& name, float *var, float std,
-				   float lower, float upper, float step) {
-	FloatParameter *p = new FloatParameter(id, name, Parameter::Continuous, true, var, std, lower, upper, step, true, replace_mode);
+				   float lower, float upper, float step, bool midi=true) {
+	FloatParameter *p = new FloatParameter(id, name, Parameter::Continuous, true, var, std, lower, upper, step, midi, replace_mode);
 	insert(p);
 	return p;
     }
@@ -564,19 +583,19 @@ class ParamMap: boost::noncopyable {
 	insert(p);
 	return p;
     }
-    inline FloatParameter *reg_par(const string& id, const string& name, float *var, float std = 0) {
-	FloatParameter *p = new FloatParameter(id, name, Parameter::Switch, true, var, std, 0, 1, 1, true, replace_mode);
+    inline FloatParameter *reg_par(const string& id, const string& name, float *var, float std = 0, bool midi = true) {
+	FloatParameter *p = new FloatParameter(id, name, Parameter::Switch, true, var, std, 0, 1, 1, midi, replace_mode);
 	insert(p);
 	return p;
     }
-    inline BoolParameter *reg_par(const string& id, const string& name, bool *var, bool std=false, bool preset=true) {
-	BoolParameter * p = new BoolParameter(id, name, Parameter::Switch, preset, var, std, true);
+    inline BoolParameter *reg_par(const string& id, const string& name, bool *var, bool std=false, bool preset=true, bool midi=true) {
+	BoolParameter * p = new BoolParameter(id, name, Parameter::Switch, preset, var, std, midi);
 	insert(p);
 	return p;
     }
     inline EnumParameter *reg_enum_par(const string& id, const string& name,
-				       const value_pair *vl, int *var, int std = 0) {
-	EnumParameter *p = new EnumParameter(id, name, vl, true, var, std, true);
+				       const value_pair *vl, int *var, int std = 0, bool midi=true) {
+	EnumParameter *p = new EnumParameter(id, name, vl, true, var, std, midi);
 	insert(p);
 	return p;
     }
@@ -589,8 +608,8 @@ class ParamMap: boost::noncopyable {
     }
     inline FloatEnumParameter *reg_enum_par(const string& id, const string& name,
 					    const value_pair *vl, float *var,
-					    int std = 0, int low = 0) {
-	FloatEnumParameter *p = new FloatEnumParameter(id, name, vl, true, var, std, low, true, replace_mode);
+					    int std = 0, int low = 0, bool midi=true) {
+	FloatEnumParameter *p = new FloatEnumParameter(id, name, vl, true, var, std, low, midi, replace_mode);
 	insert(p);
 	return p;
     }
@@ -599,6 +618,17 @@ class ParamMap: boost::noncopyable {
 	insert(p);
 	return p;
     }
+    inline IntParameter *reg_par(const string& id, const string& name, int *var, int std,
+				 int lower, int upper, bool midi) {
+	IntParameter *p = new IntParameter(id, name, Parameter::Continuous, true, var, std, lower, upper, midi);
+	insert(p);
+	return p;
+    }
+    inline IntParameter *reg_par(const string& id, const string& name, int *var, int std, bool midi) {
+	IntParameter *p = new IntParameter(id, name, Parameter::Switch, true, var, std, 0, 1, midi);
+	insert(p);
+	return p;
+    }
     inline IntParameter *reg_non_midi_par(const string& id, int *var, bool preset, int std, int lower, int upper) {
 	IntParameter *p = new IntParameter(id, "", Parameter::None, preset, var, std, lower, upper, false);
 	insert(p);
@@ -689,7 +719,7 @@ class MidiController {
     bool hasParameter(const Parameter& p) const { return *param == p; }
     Parameter& getParameter() const { return *param; }
     static MidiController *readJSON(gx_system::JsonParser& jp, ParamMap& param);
-    bool set_midi(int n, int last_value, bool update); //RT
+    bool set_midi(int n, int last_value, int *value_set, bool update); //RT
     bool set_bpm(int n, int last_value); //RT
     bool set_trans(int n, int last_value); //RT
     void set(float v, float high) { param->midi_set(v, high, _lower, _upper); }
@@ -794,7 +824,10 @@ public:
     void update_from_controller(int ctr);
     void update_from_controllers();
     void set_midi_channel(int s);
+    int get_midi_channel() {return channel_select; }
     sigc::signal<void, int, int>& signal_midi_value_changed() { return midi_value_changed; }
+    sigc::signal<void, int, int> trigger_midi_feedback;
+    sigc::signal<void, int, int>& signal_trigger_midi_feedback() { return trigger_midi_feedback; }
     void request_midi_value_update();
 };
 
