@@ -132,7 +132,7 @@ inline T *atomic_get(T*& p) {
 
 template <class T>
 inline bool atomic_compare_and_exchange(T **p, T *oldv, T *newv) {
-    return g_atomic_pointer_compare_and_exchange(reinterpret_cast<void* volatile*>(p), oldv, newv);
+    return g_atomic_pointer_compare_and_exchange(reinterpret_cast<void* volatile*>(p), static_cast<void*>(oldv), newv);
 }
 
 
@@ -272,16 +272,17 @@ inline void measure_stop()  {}
 /****************************************************************/
 
 class SkinHandling {
-private:
-    Glib::ustring empty;
 public:
-    std::vector<Glib::ustring>   skin_list;
+    Glib::ustring name;
+    std::vector<Glib::ustring> skin_list;
     SkinHandling(const std::string& styledir)
-	: skin_list() { set_styledir(styledir); }
+	: name(), skin_list() { set_styledir(styledir); }
     void set_styledir(const std::string& styledir);
-    bool is_in_list(const std::string& name);
+    bool is_in_list(const std::string& skin_name);
     const Glib::ustring& operator[](unsigned int idx);
-    unsigned int index(const Glib::ustring& name);
+    unsigned int index(const Glib::ustring& skin_name);
+    string get_cssfile() const;
+    void set_default_skin_name();
 };
 
 /****************************************************************/
@@ -371,6 +372,8 @@ public:
     std::string get_user_filepath(const std::string& basename) const { return user_dir + basename; }
     std::string get_user_ir_filepath(const std::string& basename) const { return user_IR_dir + basename; }
     std::string get_builder_filepath(const std::string& basename) const { return builder_dir + basename; }
+    void set_user_dir(std::string dir) { user_dir = dir; }
+    void set_user_IR_dir(std::string dir) { user_IR_dir = dir; }
     const std::string& get_user_dir() const { return user_dir; }
     const std::string& get_user_IR_dir() const { return user_IR_dir; }
     const std::string& get_sys_IR_dir() const { return sys_IR_dir; }
@@ -418,6 +421,8 @@ private:
     bool hideonquit;
     bool mute;
     Glib::ustring setbank;
+    Glib::ustring cmdline_bank;
+    Glib::ustring cmdline_preset;
     Glib::ustring tuner_tet;
     Glib::ustring tuner_ref;
     int sporadic_overload;
@@ -428,8 +433,6 @@ private:
     bool a_save;
     bool auto_save;
     std::string get_opskin();
-    void read_ui_vars();
-    void write_ui_vars();
 
 public:
 #ifndef NDEBUG
@@ -437,14 +440,16 @@ public:
 #endif
     SkinHandling skin;
 
+    void read_ui_vars();
+    void write_ui_vars();
     // saved in ui_rc:
+    int mainwin_visible;
     int mainwin_x;
     int mainwin_y;
     int mainwin_height;
     int window_height;
     int preset_window_height;
     int mul_buffer;
-    Glib::ustring skin_name;
     bool no_warn_latency;
     bool system_order_rack_h;
     bool system_show_value;
@@ -453,6 +458,7 @@ public:
     bool system_show_presets;
     bool system_show_toolbar;
     bool system_show_rack;
+    bool system_midiout;
     bool reload_lv2_presets;
 
 public:
@@ -461,6 +467,7 @@ public:
     void process(int argc, char** argv);
     const std::string& get_path_to_program() const { return path_to_program; }
     std::string get_style_filepath(const std::string& basename) const { return style_dir + basename; }
+    std::string get_current_style_cssfile() const { return get_style_filepath(skin.get_cssfile()); }
     std::string get_pixmap_filepath(const std::string& basename) const { return pixmap_dir + basename; }
     std::string get_preset_filepath(const std::string& basename) const { return preset_dir + basename; }
     std::string get_plugin_filepath(const std::string& basename) const { return plugin_dir + basename; }
@@ -477,7 +484,16 @@ public:
     const std::string& get_lv2_preset_dir() const { return lv2_preset_dir; }
     const std::string& get_loop_dir() const { return loop_dir; }
     const std::string& get_temp_dir() const { return temp_dir; }
+
+    void set_plugin_dir(std::string dir) { plugin_dir = dir; }
+    void set_preset_dir(std::string dir) { preset_dir = dir; }
+    void set_pluginpreset_dir(std::string dir) { pluginpreset_dir = dir; }
+    void set_lv2_preset_dir(std::string dir) { lv2_preset_dir = dir; }
+    void set_loop_dir(std::string dir) { loop_dir = dir; }
+    void set_temp_dir(std::string dir) { temp_dir = dir; }
+
     const std::string& get_factory_dir() const { return factory_dir; }
+    const std::string& get_style_dir() const { return style_dir; }
     std::string get_ladspa_config_filename() const { return get_user_filepath("ladspa_defs.js"); }
     std::string get_online_config_filename() const { return get_user_filepath("musical-artifacts.js"); }
     std::string get_online_presets_filename() const { return get_user_filepath("artifacts.js"); }
@@ -486,8 +502,13 @@ public:
     bool get_nogui() const { return nogui; }
     bool get_liveplaygui() const { return liveplaygui; }
     bool get_hideonquit() const { return hideonquit; }
+    void set_hideonquit(bool set) { hideonquit = set; }
     bool get_mute() const { return mute; }
     const Glib::ustring& get_setbank() { return setbank; }
+    void set_bank_preset(const Glib::ustring& bank, const Glib::ustring& preset) {
+        cmdline_bank = bank; cmdline_preset = preset; }
+    const Glib::ustring& get_cmdline_bank() { return cmdline_bank; }
+    const Glib::ustring& get_cmdline_preset() { return cmdline_preset; }
     const Glib::ustring& get_tuner_tet() { return tuner_tet; }
     const Glib::ustring& get_tuner_ref() { return tuner_ref; }
     int get_rpcport() const { return rpcport; }
@@ -496,6 +517,7 @@ public:
     void set_rpcaddress(const Glib::ustring& address) { rpcaddress = address; }
     const std::string& get_loadfile() const { return load_file; }
     const Glib::ustring& get_jack_instancename() const { return jack_instance; }
+    void set_jack_instancename(std::string name) {  jack_instance = name; }
     const Glib::ustring& get_jack_uuid() const { return jack_uuid; }
     const Glib::ustring& get_jack_uuid2() const { return jack_uuid2; }
     const Glib::ustring& get_jack_midi() const { return jack_midi; }
@@ -503,8 +525,11 @@ public:
     const Glib::ustring& get_jack_servername() const { return jack_servername; }
     bool get_jack_noconnect() const { return jack_noconnect; }
     bool get_jack_single() const { return jack_single; }
+    void set_jack_noconnect(bool set) { jack_noconnect = set; }
+    void set_jack_single(bool set) { jack_single = set; }
     bool get_opt_save_on_exit() const { return a_save; }
     bool get_opt_autosave() const { return auto_save; }
+    void set_opt_autosave(bool set) { auto_save = set; }
     Glib::ustring get_jack_output(unsigned int n) const;
     int get_idle_thread_timeout() const { return idle_thread_timeout; }
     int get_sporadic_overload() const { return sporadic_overload; }
