@@ -246,9 +246,14 @@ class D(Node):
     def add_count(self, tc, conn, param):
         tc["N"] += 1
     def process(self, p, conn, param, alpha):
-        Is, mUt = const = sp.symbols("Is,mUt")
-        v0, = v = sp.symbols("v:1", seq=True)
-        calc = -Is * (sp.exp(v0/mUt) - 1)
+        if 'N' in param: # new model: mUt (Vt) = thermal voltage N = Emission coefficient
+            Is, mUt, N = const = sp.symbols("Is,mUt,N")
+            v0, = v = sp.symbols("v:1", seq=True)
+            calc = -Is * (sp.exp(v0/(mUt*N)) - 1)
+        else: # old model mUt = (N * Vt)
+            Is, mUt = const = sp.symbols("Is,mUt")
+            v0, = v = sp.symbols("v:1", seq=True)
+            calc = -Is * (sp.exp(v0/mUt) - 1)
         calc = calc.subs(dict([(k,param[str(k)]) for k in const]))
         idx = p.new_row("N", self)
         p.add_2conn("Nl", idx, conn)
@@ -261,9 +266,14 @@ class D2(Node):
     def add_count(self, tc, conn, param):
         tc["N"] += 1
     def process(self, p, conn, param, alpha):
-        Is, mUt = const = sp.symbols("Is,mUt")
-        v0, = v = sp.symbols("v:1", seq=True)
-        calc = -2 * Is * sp.sinh(v0/mUt)
+        if 'N' in param: # new model: mUt (Vt) = thermal voltage N = Emission coefficient
+            Is, mUt, N = const = sp.symbols("Is,mUt,N")
+            v0, = v = sp.symbols("v:1", seq=True)
+            calc = -2 * Is * sp.sinh(v0/(mUt*N))
+        else: # old model mUt = (N * Vt)
+            Is, mUt = const = sp.symbols("Is,mUt")
+            v0, = v = sp.symbols("v:1", seq=True)
+            calc = -2 * Is * sp.sinh(v0/mUt)
         calc = calc.subs(dict([(k,param[str(k)]) for k in const]))
         idx = p.new_row("N", self)
         p.add_2conn("Nl", idx, (conn[0], conn[1]))
@@ -471,7 +481,37 @@ class Trans_L(Node):
             p.add_S(idx, conn[2*i:], 1)
             p.S[idx, end] += param["windings"][i]
         idx = p.new_row("V", self, "phi")
-        p.S[idx, start:end-1] += np.array(param["windings"])
+        vps = np.array(p.S[idx, start:end-1])
+        vps += np.array(np.array(param["windings"]))
+        p.S[idx, start:end-1] = vps
+        p.S[idx, end-1] += param["R"]
+        idx_v = p.new_row("V", self, "v")
+        p.S[idx_v, idx_v] += 1
+        p.S[idx_v, idx] += alpha
+        idx_xv = p.new_row("X", self, "v")
+        p.N["Xl"][idx_xv, idx] = alpha
+        p.N["Xr"][idx_xv, idx_v] = 1
+        p.Z[idx_xv] = 1
+
+class SingleTrans_L(Node):
+    def __init__(self, n=None, nw=2):
+        self.nw = nw
+        Node.__init__(self, "STL", n)
+    def add_count(self, tc, conn, param):
+        tc["X"] += 1
+        tc["V"] += self.nw + 2
+    def process(self, p, conn, param, alpha):
+        start = p.current_row("V") + 1
+        end = start + self.nw + 1
+        for i in range(self.nw):
+            s = "W%d" % (i+1)
+            idx = p.new_row("V", self, s)
+            p.add_S(idx, conn[2*i:], 1)
+            p.S[idx, end] += param["windings"][i]
+        idx = p.new_row("V", self, "phi")
+        vps = np.array(p.S[idx, start:end-1])
+        vps += np.array(np.array(param["windings"]))
+        p.S[idx, start:end-1] = vps
         p.S[idx, end-1] += param["R"]
         idx_v = p.new_row("V", self, "v")
         p.S[idx_v, idx_v] += 1
@@ -508,7 +548,9 @@ class Trans_F(Node):
             p.add_S(idx, conn[2*i:], 1)
             p.S[idx, end] += param["windings"][i]
         idx = p.new_row("V", self, "phi")
-        p.S[idx, start:end-1] += np.array(param["windings"])
+        vps = np.array(p.S[idx, start:end-1])
+        vps += np.array(np.array(param["windings"]))
+        p.S[idx, start:end-1] = vps
         idx_v = p.new_row("V", self, "v")
         p.S[idx_v, idx_v] += 1
         p.S[idx_v, idx] += alpha
